@@ -36,11 +36,14 @@ int
 version_cmd(cp)
 char	*cp;	/* the command line argument */
 {
+	int		rv;	/* return value */
+
 #if	LIBRARY
 	free(result_str);
 	result_str = strdup(VERSION);
 #endif
-	return version_report();
+	rv = version_report();
+	return rv;
 }
 
 /*
@@ -49,7 +52,7 @@ char	*cp;	/* the command line argument */
 int
 version_report()
 {
-	fprintf(gfp, _("Mathomatic version %s, written by George Gesslein II.\nCompile flags used: "), VERSION);
+	fprintf(gfp, _("Mathomatic version %s\nCompile flags used: "), VERSION);
 #if	__STDC__
 	fprintf(gfp, "__STDC__ ");
 #endif
@@ -77,7 +80,7 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, _("\nMaximum memory usage: %lld kilobytes\n"), (long long) (N_EQUATIONS + 3LL) * n_tokens * sizeof(token_type) * 2LL / 1000LL);
+	fprintf(gfp, _("\nMaximum memory usage: %lld kilobytes\n"), ((long long) (N_EQUATIONS + 3LL) * n_tokens * sizeof(token_type) * 2LL) / 1000);
 	return true;
 }
 
@@ -88,28 +91,81 @@ int
 solve_cmd(cp)
 char	*cp;
 {
-	int	i;
-	char	buf[MAX_CMD_LEN];
+	int		i, j;
+	char		buf[MAX_CMD_LEN];
+	long		v;	/* variable */
+	token_type	want;
+	int		diff_sign;
+	int		verify_flag = false, plural_flag = false, once_through;
 
 	if (cur_equation < 0 || cur_equation >= n_equations || n_lhs[cur_equation] <= 0 || n_rhs[cur_equation] <= 0) {
 		error(_("No current equation."));
 		return false;
 	}
 	i = next_espace();
+	if (strcmp_tospace(cp, "verify") == 0) {
+		verify_flag = true;
+		cp = skip_param(cp);
+	}
 	if (*cp == '\0') {
-		my_strlcpy(prompt_str, _("Enter variable or 0: "), sizeof(prompt_str));
+		my_strlcpy(prompt_str, _("Enter variable to solve for: "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
 	}
-	if (parse_equation(i, cp)) {
-		if (n_lhs[i] && n_rhs[i]) {
-			error(_("Invalid argument."));
-		} else {
-			if (solve(i, cur_equation)) {
+	if (verify_flag) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL || extra_characters(cp)) {
+			return false;
+		}
+		copy_espace(cur_equation, i);
+		want.level = 1;
+		want.kind = VARIABLE;
+		want.token.variable = v;
+		if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) > 0) {
+			return_result(cur_equation);
+			if (compare_es(cur_equation, i)) {
+				warning(_("Nothing was done, so no need to verify."));
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+				return true;
+			}
+			for (j = 0; j < n_rhs[cur_equation]; j += 2) {
+				if (rhs[cur_equation][j].kind == VARIABLE && (rhs[cur_equation][j].token.variable & VAR_MASK) == SIGN) {
+					plural_flag = true;
+					break;
+				}
+			}
+			subst_var_with_exp(lhs[i], &n_lhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
+			subst_var_with_exp(rhs[i], &n_rhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
+			calc_simp(lhs[i], &n_lhs[i]);
+			calc_simp(rhs[i], &n_rhs[i]);
+			once_through = false;
+check_result:
+			if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
+				if (plural_flag)
+					printf(_("Solutions verified.\n"));
+				else
+					printf(_("Solution verified.\n"));
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+				return true;
+			} else {
+				if (!once_through) {
+					symb_flag = true;
+					simpa_side(lhs[i], &n_lhs[i], true, false);
+					simpa_side(rhs[i], &n_rhs[i], true, false);
+					symb_flag = false;
+					once_through = true;
+					goto check_result;
+				}
+				error(_("Solution may be incorrect."));
+				return false;
+			}
+		}
+	} else {
+		if (parse_equation(i, cp)) {
+			if (solve_espace(i, cur_equation)) {
 				return return_result(cur_equation);
 			}
 		}
@@ -148,7 +204,7 @@ char	*cp;		/* the command line */
 int	product_flag;	/* true for product, otherwise sum */
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;			/* variable */
 	double		start, end, step = 1.0;
 	int		result_en;
 	int		n, ns;
@@ -190,10 +246,9 @@ int	product_flag;	/* true for product, otherwise sum */
 	}
 	if (*cp) {
 		if (*cp == '=') {
-			cp1 = skip_space(cp + 1);
-		} else {
-			cp1 = cp;
+			cp++;
 		}
+		cp1 = cp;
 	} else {
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), "%s = ", var_str);
@@ -225,7 +280,7 @@ int	product_flag;	/* true for product, otherwise sum */
 	if (*cp) {
 		cp1 = cp;
 		step = fabs(strtod(cp1, &cp));
-		if (step <= 0.0 || step >= MAX_K_INTEGER || cp1 == cp) {
+		if (cp1 == cp || step <= 0.0 || step >= MAX_K_INTEGER) {
 			error(_("Invalid step."));
 			return false;
 		}
@@ -234,7 +289,7 @@ int	product_flag;	/* true for product, otherwise sum */
 		return false;
 	count_down = (end < start);
 	if (fmod(fabs(start - end) / step, 1.0)) {
-		error(_("Warning: end value not reached."));
+		warning(_("End value not reached."));
 	}
 	if (product_flag) {
 		dest[0] = one_token;
@@ -353,7 +408,7 @@ int		*np;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;	/* found sub-expression flags */
-	long	v;
+	long	v;			/* variable */
 	char	temp_buf[50];
 
 	if (*np <= 0) {
@@ -589,7 +644,7 @@ display_current_directory()
 }
 
 /*
- * Output the current set options in a format suitable for RC_FILE.
+ * Output the current set options in a format suitable for reading back in.
  */
 void
 output_options()
@@ -654,17 +709,24 @@ output_options()
 }
 
 /*
- * Return true if "cp" points to a negative word.
+ * Skip over a yes/no indicator and return true if *cpp pointed to a negative word.
  */
 int
-is_no(cp)
-char	*cp;
+skip_no(cpp)
+char	**cpp;
 {
-	if (strcmp_tospace(cp, "no") == 0
-	    || strcmp_tospace(cp, "off") == 0
-	    || strcmp_tospace(cp, "false") == 0) {
+	if (strcmp_tospace(*cpp, "no") == 0
+	    || strcmp_tospace(*cpp, "not") == 0
+	    || strcmp_tospace(*cpp, "off") == 0
+	    || strcmp_tospace(*cpp, "false") == 0) {
+		*cpp = skip_param(*cpp);
 		return true;
 	}
+	if (strcmp_tospace(*cpp, "yes") == 0
+	    || strcmp_tospace(*cpp, "on") == 0
+	    || strcmp_tospace(*cpp, "true") == 0) {
+		*cpp = skip_param(*cpp);
+	}
 	return false;
 }
 
@@ -681,15 +743,10 @@ char	*cp;
 	int	negate;
 	char	*cp1;
 
-next_option:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-	negate = is_no(cp);
-	if (negate) {
-		cp = skip_param(cp);
-	}
 #if	!SECURE
 	if (strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
@@ -707,6 +764,7 @@ next_option:
 		return true;
 	}
 #endif
+	negate = skip_no(&cp);
 #if	!SILENT
 	if (strncasecmp(cp, "debug", 5) == 0) {
 		cp = skip_param(cp);
@@ -721,7 +779,7 @@ next_option:
 			cp = cp1;
 			debug_level = i;
 		}
-		goto next_option;
+		goto check_return;
 	}
 #endif
 	if (strncasecmp(cp, "special", 7) == 0) {
@@ -733,9 +791,9 @@ next_option:
 				error(_("Please specify a single character."));
 				return false;
 			}
-			special_variable_character = *cp;
+			special_variable_character = *cp++;
 		}
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
@@ -749,49 +807,49 @@ next_option:
 			cp = cp1;
 			screen_columns = i;
 		}
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "autosolve", 9) == 0) {
 		cp = skip_param(cp);
 		autosolve = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "autocalc", 8) == 0) {
 		cp = skip_param(cp);
 		autocalc = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "display2d", 7) == 0) {
 		cp = skip_param(cp);
 		display2d = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
 		cp = skip_param(cp);
 		preserve_roots = !negate;
-		goto next_option;
+		goto check_return;
 	}
-	if (strncasecmp(cp, "true_modulus", 4) == 0) {
+	if (strncasecmp(cp, "true_modulus", 8) == 0) {
 		cp = skip_param(cp);
 		true_modulus = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
 		cp = skip_param(cp);
 		color_flag = !negate;
 		cur_color = -1;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "bold", 4) == 0) {
 		cp = skip_param(cp);
 		bold_colors = !negate;
 		cur_color = -1;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "finance", 7) == 0) {
 		cp = skip_param(cp);
@@ -800,29 +858,39 @@ next_option:
 		} else {
 			i = decstrtol(cp, &cp1);
 			if (cp == cp1) {
-				i = 2;
+				if (*cp1 == '\0') {
+					i = 2;
+				} else {
+					error(_("Please specify the number of digits."));
+					return false;
+				}
 			}
-			if (i > 50) {
-				error(_("Maximum is 50."));
+			if (i < 0 || i > 50) {
+				error(_("Minimum is 0, maximum is 50."));
 				return false;
 			}
 			cp = cp1;
 			finance_option = i;
 		}
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "factor_integers", 6) == 0) {
 		cp = skip_param(cp);
 		factor_int_flag = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "right_associative_power", 5) == 0) {
 		cp = skip_param(cp);
 		right_associative_power = !negate;
-		goto next_option;
+		goto check_return;
 	}
 	error(_("Unknown set option."));
 	return false;
+
+check_return:
+	if (extra_characters(cp))
+		return false;
+	return true;
 }
 
 /*
@@ -911,7 +979,9 @@ char	*cp;
 		if (exists[i1]) {
 			k = next_espace();
 			copy_espace(i1, k);
-			return_result(k);
+			if (!return_result(k)) {
+				return false;
+			}
 		}
 	}
 	return true;
@@ -931,7 +1001,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	int		has_imag, has_real;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
-	long		v;
+	long		v;			/* variable */
 
 	if (current_not_defined()) {
 		return false;
@@ -1112,11 +1182,10 @@ int
 calculate_cmd(cp)
 char	*cp;
 {
-	int		i, j, k;
-	long		v, last_v, it_v = 0;
+	int		i, k1, k;
+	long		v, last_v, it_v = 0;	/* variables */
 	long		counter, counter_max;
-	sign_array_type	sa_mark;
-	sign_array_type	sa_value;
+	sign_array_type	sa_mark, sa_value;
 	long		l, iterations = 1;
 	token_type	*source;
 	int		n;
@@ -1162,7 +1231,7 @@ char	*cp;
 		}
 		iterations = decstrtol(cp, &cp);
 		if (*cp || iterations <= 0) {
-			error(_("Invalid number of iterations."));
+			error(_("Positive integer required."));
 			return false;
 		}
 	}
@@ -1173,11 +1242,11 @@ calc_again:
 	last_v = 0;
 	for (;;) {
 		v = -1;
-		for (j = 0; j < n; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if (source[j].token.variable > last_v
-				    && (v == -1 || source[j].token.variable < v))
-					v = source[j].token.variable;
+		for (k1 = 0; k1 < n; k1 += 2) {
+			if (source[k1].kind == VARIABLE) {
+				if (source[k1].token.variable > last_v
+				    && (v == -1 || source[k1].token.variable < v))
+					v = source[k1].token.variable;
 			}
 		}
 		if (v == -1)
@@ -1189,29 +1258,26 @@ calc_again:
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
 		if (!get_expr(tlhs, &n_tlhs)) {
-			if (repeat_flag) {
-				return true;
-			}
 			continue;
 		}
 		value_entered = true;
-		/* disguise all variables in the entered expression by making them negative: */
-		for (j = 0; j < n_tlhs; j += 2)
-			if (tlhs[j].kind == VARIABLE)
-				tlhs[j].token.variable = -tlhs[j].token.variable;
+		/* Disguise all variables in the entered expression by making them negative. */
+		/* That way they won't be improperly substituted. */
+		for (k1 = 0; k1 < n_tlhs; k1 += 2)
+			if (tlhs[k1].kind == VARIABLE)
+				tlhs[k1].token.variable = -tlhs[k1].token.variable;
 		subst_var_with_exp(trhs, &n_trhs, tlhs, n_tlhs, v);
 	}
-	/* restore disguised variables: */
-	for (j = 0; j < n_trhs; j += 2)
-		if (trhs[j].kind == VARIABLE && trhs[j].token.variable < 0)
-			trhs[j].token.variable = -trhs[j].token.variable;
+	/* Restore disguised variables: */
+	for (k1 = 0; k1 < n_trhs; k1 += 2)
+		if (trhs[k1].kind == VARIABLE && trhs[k1].token.variable < 0)
+			trhs[k1].token.variable = -trhs[k1].token.variable;
 	if (it_v) {
+		/* Handle the iteration option, where the simplified result is repeatedly plugged into variable it_v. */
 		list_var(it_v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
 		while (!get_expr(tes, &n_tes)) {
-			if (repeat_flag) {
-				return true;
-			}
+			return repeat_flag;
 		}
 		value_entered = true;
 		calc_simp(tes, &n_tes);
@@ -1232,15 +1298,16 @@ calc_again:
 	}
 	calc_simp(trhs, &n_trhs);
 	fprintf(gfp, "\n");
-	for (j = 0; j < ARR_CNT(sa_mark); j++)
-		sa_mark[j] = false;
-	for (j = 0; j < n_trhs; j += 2) {
-		if (trhs[j].kind == VARIABLE && (trhs[j].token.variable & VAR_MASK) == SIGN) {
-			sa_mark[(trhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+
+	/* Now substitute all sign variables with +1 and -1. */
+	CLEAR_ARRAY(sa_mark);
+	for (k1 = 0; k1 < n_trhs; k1 += 2) {
+		if (trhs[k1].kind == VARIABLE && (trhs[k1].token.variable & VAR_MASK) == SIGN) {
+			sa_mark[(trhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 		}
 	}
-	for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
-		if (sa_mark[j]) {
+	for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+		if (sa_mark[k1]) {
 			k++;
 		}
 	}
@@ -1248,33 +1315,33 @@ calc_again:
 	for (counter = 0; counter <= counter_max; counter++) {
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
-		for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
-			if (sa_mark[j]) {
-				sa_value[j] = (((1L << k) & counter) != 0);
+		for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+			if (sa_mark[k1]) {
+				sa_value[k1] = (((1L << k) & counter) != 0);
 				k++;
 			}
 		}
-		for (j = 0; j < n_tlhs; j += 2) {
-			if (tlhs[j].kind == VARIABLE && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
-				if (sa_value[(tlhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
-					tlhs[j].kind = CONSTANT;
-					tlhs[j].token.constant = -1.0;
+		for (k1 = 0; k1 < n_tlhs; k1 += 2) {
+			if (tlhs[k1].kind == VARIABLE && (tlhs[k1].token.variable & VAR_MASK) == SIGN) {
+				if (sa_value[(tlhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+					tlhs[k1].kind = CONSTANT;
+					tlhs[k1].token.constant = -1.0;
 				} else {
-					tlhs[j].kind = CONSTANT;
-					tlhs[j].token.constant = 1.0;
+					tlhs[k1].kind = CONSTANT;
+					tlhs[k1].token.constant = 1.0;
 				}
 			}
 		}
-		for (j = 0, k = false; j < ARR_CNT(sa_mark); j++) {
-			if (sa_mark[j]) {
+		for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
+			if (sa_mark[k1]) {
 				if (k) {
 					fprintf(gfp, ", ");
 				} else {
-					fprintf(gfp, _("Solution #%ld with "), counter + 1);
+					fprintf(gfp, _("Solution number %ld with "), counter + 1);
 				}
-				list_var((long) SIGN + (((long) j) << VAR_SHIFT), 0);
+				list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
 				fprintf(gfp, "%s = ", var_str);
-				if (sa_value[j]) {
+				if (sa_value[k1]) {
 					fprintf(gfp, "-1");
 				} else {
 					fprintf(gfp, "1");
@@ -1293,7 +1360,7 @@ calc_again:
 			list_proc(lhs[i], n_lhs[i], false);
 			fprintf(gfp, " = ");
 		}
-		list_fact(tlhs, &n_tlhs, factor_flag);
+		list_factor(tlhs, &n_tlhs, factor_flag);
 		fprintf(gfp, "\n\n");
 	}
 	if (value_entered && repeat_flag)
@@ -1346,28 +1413,43 @@ int	*diff_signp;
 		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
-	sign_flag = true;
+	sign_cmp_flag = true;
 	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], diff_signp);
-	sign_flag = false;
+	sign_cmp_flag = false;
 	return rv;
 }
 
-static int
-compare_lhs(i, j)
-int	i, j;
+/*
+ * Compare two equation spaces.
+ *
+ * Return true if identical.
+ */
+int
+compare_es(i, j)
+int	i, j;	/* equation space numbers */
 {
 	int	rv;
 	int	diff_sign;
 
-	rv = se_compare(lhs[i], n_lhs[i], lhs[i], n_lhs[i], &diff_sign);
-	if (!rv || diff_sign) {
-		error(_("Error in compare function or too many terms to compare."));
+	if (n_lhs[i] == 0 || n_lhs[j] == 0)
 		return false;
-	}
-	sign_flag = true;
+	/* compare the two left hand sides */
+	sign_cmp_flag = true;
 	rv = se_compare(lhs[i], n_lhs[i], lhs[j], n_lhs[j], &diff_sign);
-	sign_flag = false;
-	return(rv && !diff_sign);
+	sign_cmp_flag = false;
+	if (!rv || diff_sign)
+		return false;
+	if (n_rhs[i] == 0 && n_rhs[j] == 0)
+		return true;	/* two expressions, not equations */
+	if (n_rhs[i] == 0 || n_rhs[j] == 0)
+		return false;	/* mix of expression and equation */
+	/* compare the two right hand sides */
+	sign_cmp_flag = true;
+	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], &diff_sign);
+	sign_cmp_flag = false;
+	if (!rv || diff_sign)
+		return false;
+	return true;
 }
 
 /*
@@ -1379,7 +1461,6 @@ char	*cp;
 {
 	int		i, j;
 	int		diff_sign;
-	int		already_solved;
 
 	i = decstrtol(cp, &cp) - 1;
 	if (not_defined(i)) {
@@ -1395,14 +1476,12 @@ char	*cp;
 		error(_("Cannot compare an equation with itself."));
 		return false;
 	}
-#if	!SILENT
 	fprintf(gfp, _("Comparing #%d with #%d...\n"), i + 1, j + 1);
-#endif
 	if (n_rhs[i] == 0 || n_rhs[j] == 0) {
 		if (n_rhs[i] == 0 && n_rhs[j] == 0) {
 			simp_loop(lhs[i], &n_lhs[i]);
 			simp_loop(lhs[j], &n_lhs[j]);
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
@@ -1415,13 +1494,13 @@ char	*cp;
 				list_sub(j);
 			}
 #endif
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
 			uf_simp(lhs[i], &n_lhs[i]);
 			uf_simp(lhs[j], &n_lhs[j]);
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
@@ -1431,8 +1510,11 @@ char	*cp;
 		error(_("Cannot compare an expression with an equation."));
 		return false;
 	}
-	already_solved = (solved_equation(i) && solved_equation(j));
-	if (already_solved) {
+	if (compare_es(i, j)) {
+		fprintf(gfp, _("Equations are identical.\n"));
+		return true;
+	}
+	if (solved_equation(i) && solved_equation(j)) {
 		simp_loop(rhs[i], &n_rhs[i]);
 		simp_loop(rhs[j], &n_rhs[j]);
 		if (compare_rhs(i, j, &diff_sign)) {
@@ -1457,11 +1539,8 @@ char	*cp;
 		}
 	}
 	debug_string(0, _("Solving both equations for zero and unfactoring..."));
-	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
-	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
-		error(_("Can't solve for zero!"));
-		return false;
-	}
+	solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]);
+	solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]);
 	uf_simp(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
@@ -1475,11 +1554,8 @@ char	*cp;
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
-	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
-		error(_("Can't solve for zero!"));
-		return false;
-	}
+	solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]);
+	solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]);
 	uf_simp(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
@@ -1512,10 +1588,10 @@ int
 divide_cmd(cp)
 char	*cp;
 {
-	long		v, v_tmp;
+	long		v, v_tmp;		/* variables */
 	int		i, j;
 	int		nl, nr;
-	double		d1, d2, d3, d4;
+	double		d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
 	v = 0;
@@ -1537,16 +1613,23 @@ char	*cp;
 		return false;
 	}
 	fprintf(gfp, "\n");
-	calc_simp(lhs[i], &nl);
 	calc_simp(rhs[i], &nr);
+	calc_simp(lhs[i], &nl);
+	if (get_constant(lhs[i], nl, &d2)) {
+		check_divide_by_zero(d2);
+	}
 	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
-		d4 = modf(d1 / d2, &d3);
-		fprintf(gfp, _("Result of numerical division: %.*g/%.*g = %.*g\n"), precision, d1, precision, d2, precision, d1 / d2);
-		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d3, precision, d4 * d2);
+		d3 = gcd_verified(d1, d2);
+		d4 = modf(d1 / d2, &d5);
+		if (d3 == 0 || d3 == 1) {
+			fprintf(gfp, "%.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d2);
+		} else {
+			fprintf(gfp, "%.*g/%.*g = %.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d3, precision, d2 / d3, precision, d1 / d2);
+		}
+		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d5, precision, d4 * d2);
 		d1 = fabs(d1);
 		d2 = fabs(d2);
-		d3 = gcd_verified(d1, d2);
-		if (d3 <= 0.0) {
+		if (d3 == 0.0) {
 			fprintf(gfp, _("No GCD found.\n"));
 			return true;
 		}
@@ -1577,9 +1660,11 @@ char	*cp;
 		list_var(v_tmp, 0);
 		fprintf(gfp, _("Polynomial division successful using base variable (%s).\n"), var_str);
 		fprintf(gfp, _("The quotient is:\n"));
-		list_fact(tlhs, &n_tlhs, false);
+		display_fractions_and_group(tlhs, &n_tlhs);
+		list_factor(tlhs, &n_tlhs, false);
 		fprintf(gfp, _("\n\nThe remainder is:\n"));
-		list_fact(trhs, &n_trhs, false);
+		display_fractions_and_group(trhs, &n_trhs);
+		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("Polynomial division failed.\n"));
@@ -1591,8 +1676,9 @@ char	*cp;
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
-		fprintf(gfp, _("Polynomial GCD (Euclidean algorithm iterations = %d):\n"), j);
-		list_fact(trhs, &n_trhs, false);
+		fprintf(gfp, _("Polynomial GCD (iterations = %d):\n"), j);
+		display_fractions_and_group(trhs, &n_trhs);
+		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
 	} else {
 		fprintf(gfp, _("No univariate polynomial GCD found.\n"));
@@ -1608,19 +1694,18 @@ int
 eliminate_cmd(cp)
 char	*cp;
 {
-	long	v, last_v, v1, va[MAX_VARS];
+	long	v, last_v, v1, va[MAX_VARS];		/* variables */
+	int	vc = 0;					/* variable count */
 	int	i, n;
 	int	success_flag = false, did_something = false, repeat_flag = false;
 	char	used[N_EQUATIONS];
-	int	vc = 0;
 	char	*cp_start;
 
 	if (*cp == '\0') {
 		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
 		return false;
 	}
-	for (i = 0; i < ARR_CNT(used); i++)
-		used[i] = false;
+	CLEAR_ARRAY(used);
 	if (current_not_defined()) {
 		return false;
 	}
@@ -1734,8 +1819,8 @@ next_var:
 
 static int
 elim_sub(i, v)
-int	i;
-long	v;
+int	i;	/* equation number */
+long	v;	/* variable */
 {
 	token_type	want;
 
@@ -1790,7 +1875,7 @@ char	*cp;
 				return(result_str != NULL);
 			}
 #endif
-			flist_sub(i);
+			flist_equation(i);
 		}
 	}
 	return true;
@@ -1850,17 +1935,24 @@ char	*cp;
 	int	int_flag = false, displayed = false;
 	char	*cp1;
 
-	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
-		cp = skip_param(cp);
-	} else if (strcmp_tospace(cp, "java") == 0) {
-		cp = skip_param(cp);
-		language = 1;
-	} else if (strcmp_tospace(cp, "python") == 0) {
-		cp = skip_param(cp);
-		language = 2;
-	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
-		cp = skip_param(cp);
-		int_flag = true;
+	for (;; cp = skip_param(cp)) {
+		if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
+			language = 0;
+			continue;
+		}
+		if (strcmp_tospace(cp, "java") == 0) {
+			language = 1;
+			continue;
+		}
+		if (strcmp_tospace(cp, "python") == 0) {
+			language = 2;
+			continue;
+		}
+		if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
+			int_flag = true;
+			continue;
+		}
+		break;
 	}
 	do {
 		cp1 = cp;
@@ -1874,22 +1966,27 @@ char	*cp;
 		for (; i <= j; i++) {
 			if (n_lhs[i] > 0) {
 				if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-					error(_("Not a solved equation."));
-					continue;
+					warning(_("Can't make assignment statement because this is not an equation."));
 				}
 				approximate_roots = true;
-				simp_i(rhs[i], &n_rhs[i]);
-				approximate_roots = false;
+				simp_i(lhs[i], &n_lhs[i]);
 				if (int_flag) {
-					uf_repeat_always(rhs[i], &n_rhs[i]);
+					uf_repeat_always(lhs[i], &n_lhs[i]);
 				}
+				if (n_rhs[i] > 0) {
+					simp_i(rhs[i], &n_rhs[i]);
+					if (int_flag) {
+						uf_repeat_always(rhs[i], &n_rhs[i]);
+					}
+				}
+				approximate_roots = false;
 				make_fractions_and_group(i);
-				if (int_flag && !int_expr(rhs[i], n_rhs[i])) {
-					error(_("Not an integer equation."));
-					continue;
+				if (int_flag && (!int_expr(lhs[i], n_lhs[i]) || !int_expr(rhs[i], n_rhs[i]))) {
+					warning(_("Not an integer expression."));
+				}
+				if (list_c_equation(i, language, int_flag) > 0) {
+					displayed = true;
 				}
-				list_c_equation(i, language, int_flag);
-				displayed = true;
 			}
 		}
 	} while (*cp);
@@ -1905,8 +2002,8 @@ char	*cp;
 {
 	int		i, j, k;
 	int		i1;
-	int		vc, cnt;
-	long		v1, last_v;
+	long		v1, last_v;		/* variables */
+	int		vc, cnt;		/* variable counts */
 	sort_type	va[MAX_VARS];
 	token_type	*p1;
 	int		n1;
@@ -2017,7 +2114,9 @@ char	*cp;
 				approximate_roots = true;
 				simp_sub(i);
 				approximate_roots = false;
-				return_result(i);
+				if (!return_result(i)) {
+					return false;
+				}
 			}
 		}
 	} while (*cp);
@@ -2033,10 +2132,10 @@ char	*cp;
 {
 	int	i, j;
 	int	n;
-	long	last_v, v;
+	long	last_v, v;	/* variables */
 	char	*cp_start;
-	long	va[MAX_VARS];
-	int	vc;
+	long	va[MAX_VARS];	/* variable array */
+	int	vc;		/* variable count */
 	int	found;
 	char	*cp1;
 
@@ -2104,14 +2203,9 @@ char	*cp;
 				}
 				cp1 = skip_param(cp);
 				input_column += (cp1 - cp_start);
-				if (!case_sensitive_flag) {
-					str_tolower(cp1);
-				}
-				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
+				if ((cp1 = parse_expr(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
-				if (extra_characters(cp1))
-					return false;
 				goto do_this;
 			}
 		}
@@ -2121,7 +2215,8 @@ char	*cp;
 			continue;
 		}
 do_this:
-		/* disguise all variables in the entered expression by making them negative: */
+		/* Disguise all variables in the entered expression by making them negative. */
+		/* That way they won't be improperly substituted. */
 		for (j = 0; j < n; j += 2) {
 			if (scratch[j].kind == VARIABLE) {
 				scratch[j].token.variable = -scratch[j].token.variable;
@@ -2130,7 +2225,7 @@ do_this:
 		subst_var_with_exp(tlhs, &n_tlhs, scratch, n, v);
 		subst_var_with_exp(trhs, &n_trhs, scratch, n, v);
 	}
-	/* restore disguised variables: */
+	/* Restore disguised variables: */
 	for (j = 0; j < n_tlhs; j += 2)
 		if (tlhs[j].kind == VARIABLE && tlhs[j].token.variable < 0)
 			tlhs[j].token.variable = -tlhs[j].token.variable;
@@ -2152,10 +2247,17 @@ int
 simplify_cmd(cp)
 char	*cp;
 {
-	int	i, j;
-	int	quick_flag = false, symb = false, frac_flag = false;
+	int		i, i1, j;
+	int		k, k1;
+	long		counter, counter_max, previous_solution_number[N_EQUATIONS];
+	sign_array_type	sa_mark, sa_value;
+	int		sign_flag = false, quick_flag = false, symb = false, frac_flag = false;
 
 	for (;; cp = skip_param(cp)) {
+		if (strncasecmp(cp, "sign", 4) == 0) {
+			sign_flag = true;
+			continue;
+		}
 		if (strncasecmp(cp, "symbolic", 4) == 0) {
 			symb = true;
 			continue;
@@ -2178,7 +2280,100 @@ char	*cp;
 		if (n_lhs[i]) {
 			simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
 			simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
-			return_result(i);
+			if (!return_result(i)) {
+				symb_flag = false;
+				return false;
+			}
+			if (!sign_flag)
+				continue;
+			/* Now substitute all sign variables with +1 and -1. */
+			CLEAR_ARRAY(previous_solution_number);
+			CLEAR_ARRAY(sa_mark);
+			for (k1 = 0; k1 < n_lhs[i]; k1 += 2) {
+				if (lhs[i][k1].kind == VARIABLE && (lhs[i][k1].token.variable & VAR_MASK) == SIGN) {
+					sa_mark[(lhs[i][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+				}
+			}
+			for (k1 = 0; k1 < n_rhs[i]; k1 += 2) {
+				if (rhs[i][k1].kind == VARIABLE && (rhs[i][k1].token.variable & VAR_MASK) == SIGN) {
+					sa_mark[(rhs[i][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+				}
+			}
+			for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+				if (sa_mark[k1]) {
+					k++;
+				}
+			}
+			if (k == 0)
+				continue;
+			counter_max = (1L << k) - 1L;
+			for (counter = 0; counter <= counter_max; counter++) {
+				i1 = next_espace();
+				copy_espace(i, i1);
+				for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+					if (sa_mark[k1]) {
+						sa_value[k1] = (((1L << k) & counter) != 0);
+						k++;
+					}
+				}
+				for (k1 = 0; k1 < n_lhs[i1]; k1 += 2) {
+					if (lhs[i1][k1].kind == VARIABLE && (lhs[i1][k1].token.variable & VAR_MASK) == SIGN) {
+						if (sa_value[(lhs[i1][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+							lhs[i1][k1].kind = CONSTANT;
+							lhs[i1][k1].token.constant = -1.0;
+						} else {
+							lhs[i1][k1].kind = CONSTANT;
+							lhs[i1][k1].token.constant = 1.0;
+						}
+					}
+				}
+				for (k1 = 0; k1 < n_rhs[i1]; k1 += 2) {
+					if (rhs[i1][k1].kind == VARIABLE && (rhs[i1][k1].token.variable & VAR_MASK) == SIGN) {
+						if (sa_value[(rhs[i1][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+							rhs[i1][k1].kind = CONSTANT;
+							rhs[i1][k1].token.constant = -1.0;
+						} else {
+							rhs[i1][k1].kind = CONSTANT;
+							rhs[i1][k1].token.constant = 1.0;
+						}
+					}
+				}
+				for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
+					if (sa_mark[k1]) {
+						if (k) {
+							fprintf(gfp, ", ");
+						} else {
+							fprintf(gfp, _("Solution number %ld with "), counter + 1);
+						}
+						list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
+						fprintf(gfp, "%s = ", var_str);
+						if (sa_value[k1]) {
+							fprintf(gfp, "-1");
+						} else {
+							fprintf(gfp, "1");
+						}
+						k = true;
+					}
+				}
+				if (k)
+					fprintf(gfp, ":\n");
+				simpa_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
+				simpa_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+				for (k1 = 0; k1 < ARR_CNT(previous_solution_number); k1++) {
+					if (previous_solution_number[k1]) {
+						if (compare_es(k1, i1)) {
+							n_lhs[i1] = 0;
+							n_rhs[i1] = 0;
+							fprintf(gfp, _("is identical to solution number %ld.\n"), previous_solution_number[k1]);
+							break;
+						}
+					}
+				}
+				if (n_lhs[i1]) {
+					list_sub(i1);
+					previous_solution_number[i1] = counter + 1;
+				}
+			}
 		}
 	}
 	symb_flag = false;
@@ -2193,23 +2388,26 @@ factor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	i1, count;
-	long	v;
+	int	i1;
+	long	v;				/* variable */
 	double	d;
-	char	*cp_start, buf[MAX_CMD_LEN];
-	int	valid_range = false;
-	int	repeat_flag = false;
+	char	buf[MAX_CMD_LEN];
+	int	valid_range = false, repeat_flag = false, power_flag = false;
 
-	cp_start = cp;
 	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "numbers") == 0) {
 		repeat_flag = true;
 		cp = skip_param(cp);
 	} else {
+		if (strcmp_tospace(cp, "power") == 0) {
+			power_flag = true;
+			cp = skip_param(cp);
+		}
 		valid_range = get_range(&cp, &i, &j);
 		if (!valid_range) {
-			cp = cp_start;
+			if (*cp == '\0' || power_flag)
+				return false;
 		}
 	}
 	if (!valid_range) {
@@ -2223,11 +2421,11 @@ char	*cp;
 			if (*cp == '\0') {
 				return true;
 			}
-			for (count = 1; *cp; count++) {
+			for (; *cp; ) {
 				d = strtod(cp, &cp);
 				cp = skip_space(cp);
 				if (!factor_one(d)) {
-					error(_("Not a valid integer or number too large."));
+					error(_("Not a valid integer or number too large to factor."));
 					return false;
 				}
 				display_unique();
@@ -2235,22 +2433,46 @@ char	*cp;
 		} while (repeat_flag);
 		return true;
 	}
-	v = 0;
-	do {
-		if (*cp) {
-			if ((cp = parse_var2(&v, cp)) == NULL) {
-				return false;
-			}
-		}
+	if (power_flag) {
+		if (extra_characters(cp))
+			return false;
 		for (i1 = i; i1 <= j; i1++) {
 			if (n_lhs[i1]) {
-				simpv_side(lhs[i1], &n_lhs[i1], v);
-				simpv_side(rhs[i1], &n_rhs[i1], v);
+				factor_power(lhs[i1], &n_lhs[i1]);
+				do {
+					simp_loop(lhs[i1], &n_lhs[i1]);
+				} while (factor_power(lhs[i1], &n_lhs[i1]));
+				if (n_rhs[i1]) {
+					factor_power(rhs[i1], &n_rhs[i1]);
+					do {
+						simp_loop(rhs[i1], &n_rhs[i1]);
+					} while (factor_power(rhs[i1], &n_rhs[i1]));
+				}
+				if (!return_result(i1))
+					return false;
+			}
+		}
+	} else {
+		v = 0;
+		do {
+			if (*cp) {
+				if ((cp = parse_var2(&v, cp)) == NULL) {
+					return false;
+				}
+			}
+			for (i1 = i; i1 <= j; i1++) {
+				if (n_lhs[i1]) {
+					simpv_side(lhs[i1], &n_lhs[i1], v);
+					simpv_side(rhs[i1], &n_rhs[i1], v);
+				}
+			}
+		} while (*cp);
+		for (i1 = i; i1 <= j; i1++) {
+			if (n_lhs[i1]) {
+				if (!return_result(i1))
+					return false;
 			}
 		}
-	} while (*cp);
-	for (i1 = i; i1 <= j; i1++) {
-		return_result(i1);
 	}
 	return true;
 }
@@ -2277,7 +2499,10 @@ char	*cp;
 		if (n_lhs[i]) {
 			uf_simp(lhs[i], &n_lhs[i]);
 			uf_simp(rhs[i], &n_rhs[i]);
-			return_result(i);
+			if (!return_result(i)) {
+				partial_flag = true;
+				return false;
+			}
 		}
 	}
 	partial_flag = true;
@@ -2300,7 +2525,8 @@ char	*cp;
 		if (n_lhs[i]) {
 			frac_side(lhs[i], &n_lhs[i]);
 			frac_side(rhs[i], &n_rhs[i]);
-			return_result(i);
+			if (!return_result(i))
+				return false;
 		}
 	}
 	return true;
@@ -2373,7 +2599,7 @@ FILE	*fp;
 	char	*cp;
 
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((rv = setjmp(jmp_save)) != 0) {
+	if ((rv = setjmp(jmp_save)) != 0) {	/* trap errors */
 		clean_up();
 		if (rv == 14) {
 			error(_("Expression too large."));
@@ -2388,7 +2614,7 @@ FILE	*fp;
 			}
 			set_error_level(cp);
 			if (!process(cp)) {
-				longjmp(jmp_save, 3);
+				longjmp(jmp_save, 3);	/* jump to the above error trap */
 			}
 		}
 	}
@@ -2469,7 +2695,7 @@ edit_again:
 	}
 	clear_all();
 	if (!read_cmd(cp)) {
-		if (pause_cmd(_("Prepare to run the editor"))) {
+		if (pause_cmd(_("Prepare to rerun the editor"))) {
 			goto edit_again;
 		}
 	}
