@@ -78,7 +78,7 @@ char	*cp;
 		radius_root = pow(radius, 1.0 / root);
 	}
 	check_err();
-	fprintf(gfp, _("\nThe polar coordinates are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
+	fprintf(gfp, _("\nThe polar coordinates before root taking are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
 	    radius, theta, theta * 180.0 / M_PI);
 	if (root) {
 		if (c.im == 0.0) {
@@ -178,7 +178,7 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		debug_string(0, "Warning: complex number root approximated, solutions may be lost or inaccurate.");
+		warning(_("Complex number root approximated."));
 	}
 	return modified;
 }
