@@ -1,8 +1,8 @@
 /*
- * An independent, floating point complex number library for C.
+ * An independent, floating point complex number arithmetic library for C.
  * Include "complex.h" if you use this.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "complex.h"
@@ -14,7 +14,8 @@
 #define epsilon	0.00000000000005	/* a good value for doubles */
 
 /*
- * Zero out relatively very small real or imaginary parts of a complex number.
+ * Zero out relatively very small real or imaginary parts of a complex number,
+ * because they probably are a result of accumulated floating point inaccuracies.
  *
  * Return true if something was zeroed out.
  */
@@ -65,8 +66,7 @@ complexs	a;
  */
 complexs
 complex_mult(a, b)
-complexs	a;
-complexs	b;
+complexs	a, b;
 {
 	complexs	r;
 
@@ -76,7 +76,7 @@ complexs	b;
 }
 
 /*
- * Divide complex number "a" by complex number "b" and return result.
+ * Divide two complex numbers (a / b) and return result.
  */
 complexs
 complex_div(a, b)
@@ -131,8 +131,7 @@ complexs	a;
  */
 complexs
 complex_pow(a, b)
-complexs	a;
-complexs	b;
+complexs	a, b;
 {
 	complexs	r;
 
