@@ -288,14 +288,19 @@ derivative_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;		/* variable */
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
+	int		simplify_flag = true;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	if (strcmp_tospace(cp, "nosimplify") == 0) {
+		simplify_flag = false;
+		cp = skip_param(cp);
+	}
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
 		source = rhs[cur_equation];
@@ -345,9 +350,14 @@ char	*cp;
 #if	!SILENT
 	list_var(v, 0);
 	if (n_rhs[cur_equation]) {
-		printf(_("Differentiating the RHS with respect to (%s) and simplifying...\n"), var_str);
+		printf(_("Differentiating the RHS with respect to (%s)"), var_str);
 	} else {
-		printf(_("Differentiating with respect to (%s) and simplifying...\n"), var_str);
+		printf(_("Differentiating with respect to (%s)"), var_str);
+	}
+	if (simplify_flag) {
+		printf(" and simplifying...\n");
+	} else {
+		printf("...\n");
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -358,7 +368,9 @@ char	*cp;
 			error(_("Differentiation failed."));
 			return false;
 		}
-		simpa_side(dest, &n1, true, false);
+		if (simplify_flag) {
+			simpa_side(dest, &n1, true, false);
+		}
 	}
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
@@ -377,7 +389,7 @@ extrema_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;		/* variable */
 	long		l1, order = 1;
 	token_type	want;
 	token_type	*source;
@@ -463,7 +475,7 @@ int
 taylor_cmd(cp)
 char	*cp;
 {
-	long		v = 0;
+	long		v = 0;			/* variable */
 	int		i, j, k, i1;
 	int		level;
 	long		l1, n, order = -1L;
@@ -529,20 +541,17 @@ char	*cp;
 	}
 	blt(rhs[our], source, *nps * sizeof(token_type));
 	our_nrhs = *nps;
+/* Simplify and take the first derivative: */
+	uf_simp(rhs[our], &our_nrhs);
 	if (!differentiate(rhs[our], &our_nrhs, v)) {
 		error(_("Differentiation failed."));
 		return false;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
-		if ((cp = parse_section(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
+		if ((cp = parse_expr(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
 			return false;
 		}
-		if (extra_characters(cp))
-			return false;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
@@ -633,7 +642,7 @@ loop_again:
 		dest[i1].level++;
 	uf_simp(dest, &n1);
 	if (!flag && exp_contains_infinity(dest, n1)) {
-		debug_string(0, _("Singularity warning: result invalid because it contains infinity or NaN."));
+		warning(_("Result invalid because it contains infinity or NaN."));
 		flag = true;
 	}
 	side_debug(1, dest, n1);
@@ -644,7 +653,9 @@ loop_again:
 				return false;
 			}
 		}
+		symb_flag = true;
 		simpa_side(rhs[our], &our_nrhs, true, true);
+		symb_flag = false;
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
 			i1 = n1;
 			if ((i1 + 1 + our_nrhs) > n_tokens)
@@ -680,10 +691,10 @@ limit_cmd(cp)
 char	*cp;
 {
 	int		i, j, k;
-	long		v = 0;
+	long		v = 0;			/* variable */
 	token_type	solved_v, want;
 	char		*cp_start;
-	int		infinity_flag, found_den, complex_den, num, den;
+	int		infinity_flag, found_den, complex_den, num, den;	/* flags */
 	int		level;
 
 	cp_start = cp;
@@ -727,10 +738,7 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
-		if ((cp = parse_section(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
+		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
 			return false;
 		}
 	} else {
@@ -740,35 +748,31 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_characters(cp))
-		return false;
-/* copy to a new equation space and work on the copy: */
+/* copy the current equation to a new equation space and work on the copy: */
 	copy_espace(cur_equation, i);
-	cur_equation = i;
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
 /* first fully simplify the current equation and group denominators together: */
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
-	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
+	group_proc(rhs[i], &n_rhs[i]);
 	found_den = complex_den = false;
-	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
-		switch (rhs[cur_equation][j].token.operatr) {
+	for (j = 1; j < n_rhs[i]; j += 2) {
+		switch (rhs[i][j].token.operatr) {
 		case DIVIDE:
 		case MODULUS:
 			num = den = false;
-			level = rhs[cur_equation][j].level;
+			level = rhs[i][j].level;
 			for (k = j + 2;; k += 2) {
-				if (rhs[cur_equation][k-1].kind == VARIABLE
-				    && rhs[cur_equation][k-1].token.variable == v) {
+				if (rhs[i][k-1].kind == VARIABLE && rhs[i][k-1].token.variable == v) {
 /* the limit variable was found in a denominator */
 					den = true;
 					found_den = true;
 				}
-				if (k >= n_rhs[cur_equation] || rhs[cur_equation][k].level <= level) {
+				if (k >= n_rhs[i] || rhs[i][k].level <= level) {
 					break;
 				}
-				switch (rhs[cur_equation][k].token.operatr) {
+				switch (rhs[i][k].token.operatr) {
 				case DIVIDE:
 				case MODULUS:
 /* a denominator contains a fraction */
@@ -776,9 +780,8 @@ char	*cp;
 					break;
 				}
 			}
-			for (k = j - 1; k >= 0 && rhs[cur_equation][k].level >= level; k--) {
-				if (rhs[cur_equation][k].kind == VARIABLE
-				    && rhs[cur_equation][k].token.variable == v) {
+			for (k = j - 1; k >= 0 && rhs[i][k].level >= level; k--) {
+				if (rhs[i][k].kind == VARIABLE && rhs[i][k].token.variable == v) {
 /* the limit variable was found in a numerator */
 					num = true;
 				}
@@ -792,7 +795,7 @@ char	*cp;
 	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
 		debug_string(0, "Substituting...");
-		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
+		subst_var_with_exp(rhs[i], &n_rhs[i], tes, n_tes, v);
 		goto finish_up;
 	}
 	debug_string(0, "Solving...");
@@ -809,40 +812,40 @@ char	*cp;
 		tlhs[2].kind = VARIABLE;
 		tlhs[2].token.variable = v;
 		n_tlhs = 3;
-		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
+		subst_var_with_exp(rhs[i], &n_rhs[i], tlhs, n_tlhs, v);
 	}
 /* General limit taking, solve for the limit variable: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
+	if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
 /* replace the limit variable (LHS) with the limit expression: */
-	blt(lhs[cur_equation], tes, n_tes * sizeof(token_type));
-	n_lhs[cur_equation] = n_tes;
+	blt(lhs[i], tes, n_tes * sizeof(token_type));
+	n_lhs[i] = n_tes;
 /* symbolically simplify the RHS: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
-	if (exp_contains_nan(rhs[cur_equation], n_rhs[cur_equation])) {
+	if (exp_contains_nan(rhs[i], n_rhs[i])) {
 		error(_("Unable to take limit; result contains NaN (Not a Number)."));
 		return false;
 	}
 /* solve back for the original variable: */
-	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
+	if (solve_sub(&solved_v, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
 finish_up:
 /* symbolically simplify before returning the result: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
 	symb_flag = false;
-	if (exp_contains_nan(rhs[cur_equation], n_rhs[cur_equation])) {
+	if (exp_contains_nan(rhs[i], n_rhs[i])) {
 		error(_("Unable to take limit; result contains NaN (Not a Number)."));
 		return false;
 	}
-	return return_result(cur_equation);
+	return return_result(i);
 }
