@@ -44,6 +44,7 @@ extern int		autocalc;
 extern char		special_variable_character;
 extern int		integer_coefficients;
 extern int		right_associative_power;
+extern int		power_starstar;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
@@ -55,7 +56,7 @@ extern int		partial_flag;
 extern int		symb_flag;
 extern int		high_prec;
 extern int		input_column;
-extern int		sign_flag;
+extern int		sign_cmp_flag;
 extern double		small_epsilon;
 extern double		epsilon;
 extern char		var_str[MAX_VAR_LEN+80];
@@ -72,6 +73,7 @@ extern FILE		*gfp;
 extern jmp_buf		jmp_save;
 extern int		test_mode;
 extern int		quiet_mode;
+extern volatile int	abort_flag;
 
-extern char		*error_str;
+extern const char	*error_str;
 extern char		*result_str;
