@@ -220,7 +220,7 @@ int	n;	/* equation space number */
  * List an equation side with optional integer factoring.
  */
 int
-list_fact(equation, np, factor_flag)
+list_factor(equation, np, factor_flag)
 token_type	*equation;
 int		*np;
 int		factor_flag;
