@@ -1,12 +1,13 @@
 # General factorial function written in Python.
-# Works transparently with integers and floating point.
+# Factorials are the product of all positive integers <= a given number.
+# Works transparently with integers and floating point,
+# that is it returns the same type as its single argument.
 # Gives an error for negative or non-integer input values.
 
 def fact(x):
 	"Return x! (x factorial)."
 	if (x < 0 or (x % 1.0) != 0.0):
 		raise ValueError, "Factorial argument must be a positive integer."
-		return
 	if (x == 0):
 		return x + 1
 	d = x
@@ -15,6 +16,5 @@ def fact(x):
 		temp = d * x
 		if (temp <= d):
 			raise ValueError, "Factorial result too large."
-			return
 		d = temp
 	return d
