@@ -48,7 +48,7 @@ double	d1, d2;
 	for (count = 1; count < 50; count++) {
 		r1 = fmod(larger, divisor);
 		if (r1 <= d || (divisor - r1) <= d) {
-			if (r1 != 0.0 && divisor <= (100 * d))
+			if (r1 != 0.0 && divisor <= (100.0 * d))
 				return 0.0;
 			return divisor;
 		}
@@ -71,9 +71,7 @@ double	d1, d2;
 
 	divisor = gcd(d1, d2);
 	if (divisor != 0.0) {
-		if (fmod(d1, divisor) != 0.0)
-			return 0.0;
-		if (fmod(d2, divisor) != 0.0)
+		if (gcd(d1 / divisor, d2 / divisor) != 1.0)
 			return 0.0;
 	}
 	return divisor;
@@ -102,10 +100,10 @@ double	d1;	/* value to round */
  * numerator = d / divisor
  * denominator = 1.0 / divisor
  *
- * Return true with integer numerator and denominator if conversion was successful.
+ * Return true with integers in numerator and denominator if conversion was successful.
  * Otherwise return false with numerator = "d" and denominator = "1.0".
  *
- * True return indicates "d" is rational and finite, otherwise "d" is irrational.
+ * True return indicates "d" is rational and finite, otherwise "d" is probably irrational.
  */
 int
 f_to_fraction(d, numeratorp, denominatorp)
@@ -126,17 +124,13 @@ double	*denominatorp;	/* returned denominator */
 	if (fmod(d, 1.0) == 0.0) {
 		return true;
 	}
-#if	true	/* set true for more integer oriented math */
-	k3 = fabs(d) * epsilon;
-#else
 	k3 = fabs(d) * small_epsilon;
-#endif
 	k4 = my_round(d);
 	if (k4 != 0.0 && fabs(k4 - d) <= k3) {
 		*numeratorp = k4;
 		return true;
 	}
-/* try to convert non-integer, floating point value in "d" to a fraction: */
+/* try to convert non-integer floating point value in "d" to a fraction: */
 	if ((divisor = gcd(1.0, d)) > epsilon) {
 		numerator = my_round(d / divisor);
 		denominator = my_round(1.0 / divisor);
@@ -156,12 +150,7 @@ double	*denominatorp;	/* returned denominator */
 		if (fabs(k3 - k4) > (small_epsilon * fabs(k3))) {
 			return false;	/* result is too inaccurate */
 		}
-		if (fmod(numerator, 1.0)) {
-			error("Internal error: Numerator not integral.");
-		}
-		if (fmod(denominator, 1.0)) {
-			error("Internal error: Denominator not integral.");
-		}
+/* numerator and denominator are integral */
 		*numeratorp = numerator;
 		*denominatorp = denominator;
 		return true;
