@@ -27,7 +27,7 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused." },
 #if	!LIBRARY
@@ -35,33 +35,33 @@ static com_type com_list[] = {
 #endif
 {	"compare",	NULL,			compare_cmd,		"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
 {	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the specified equation spaces." },
-{	"derivative",	"differentiate",	derivative_cmd,		"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
 {	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-range]",			"Display equation spaces in multi-line fraction format." },
 #if	!LIBRARY
-{	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
+{	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD." },
 #endif
 {	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
 #if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
 {	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"[\"repeat\"] variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	"stationary",		extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
+{	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
 {	"factor",	NULL,			factor_cmd,		"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor given integers or factor variables in expressions." },
 {	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert expressions with algebraic fractions into a single fraction." },
 {	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help." },
-{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
+{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of an expression (see real command)." },
 {	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials, order times." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
 {	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression." },
-{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single line format." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
 #if	!LIBRARY
 {	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
-{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
+{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up an equation into smaller, more efficient equations." },
 #endif
 #if	!LIBRARY
-{	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
+{	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key. Optionally display a message." },
 #endif
-{	"product",	NULL,			product_cmd,		"variable start end [step]",				"Compute the product as variable goes from start to end." },
+{	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end." },
 #if	READLINE
 {	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push specified equation spaces into the readline history." },
 #endif
@@ -71,7 +71,7 @@ static com_type com_list[] = {
 #if	!SECURE
 {	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
+{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of an expression (see imaginary command)." },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
 {	"roots",	NULL,			roots_cmd,		"root real-part imaginary-part",			"Display all the roots of a complex number." },
@@ -80,18 +80,18 @@ static com_type com_list[] = {
 {	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file." },
 #endif
 {	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Set or display various session options." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
-{	"solve",	NULL,			solve_cmd,		"variable or \"0\"",					"Solve the current equation for a variable or for zero." },
-{	"sum",		NULL,			sum_cmd,		"variable start end [step]",				"Compute the summation as variable goes from start to end." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] variable or \"0\"",			"Solve the current equation for a variable or for zero." },
+{	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end." },
 #if	!LIBRARY
 {	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show running (grand) total." },
 #endif
-{	"taylor",	NULL,			taylor_cmd,		"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
+{	"taylor",	NULL,			taylor_cmd,		"variable order point",					"Compute the Taylor series expansion of the current expression." },
 {	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) expressions." },
 #if	!LIBRARY
 {	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations." },
 #endif
-{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and maximum memory usage." }
+{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and memory usage." }
 };
 
 /*
@@ -157,9 +157,8 @@ char	*cp;
 			if (autosolve) {
 				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
 				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
-				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
-				    || (rhs[n][0].kind == VARIABLE && (rhs[n][0].token.variable & VAR_MASK) > SIGN)))) {
-					if (solve(n, cur_equation)) {
+				    || n_rhs[n] == 1) {
+					if (solve_espace(n, cur_equation)) {
 						return return_result(cur_equation);
 					}
 					return false;
@@ -173,7 +172,7 @@ char	*cp;
 			}
 			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
-				/* easy selecting of equation spaces */
+				/* easy selecting of equation spaces by just typing in the equation number */
 				n_lhs[n] = 0;
 				n = lhs[n][0].token.constant - 1;
 				goto return_ok;
@@ -181,18 +180,21 @@ char	*cp;
 #if	!LIBRARY
 			if (autocalc) {
 				/* the numerical input calculation */
-				if (!exp_is_numeric(lhs[n], n_lhs[n])) {
-					goto return_ok;		/* not numerical (contains a variable) */
+				if (n_lhs[n]) {
+					if (!exp_is_numeric(lhs[n], n_lhs[n])) {
+						goto return_ok;		/* not numerical (contains a variable) */
+					}
+					/* copy the LHS to the RHS */
+					blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
+					n_rhs[n] = n_lhs[n];
 				}
-				/* make the expression an equation */
-				blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
-				n_rhs[n] = n_lhs[n];
+				/* make the expression an equation by making the LHS the "answer" variable */
 				lhs[n][0].level = 1;
 				lhs[n][0].kind = VARIABLE;
 				parse_var(&lhs[n][0].token.variable, "answer");
 				n_lhs[n] = 1;
 				i = cur_equation;
-				/* make the current equation and run the calculate command on it */
+				/* temporarily make it the current equation and run the calculate command on it */
 				cur_equation = n;
 				calculate_cmd("");
 				cur_equation = i;
@@ -224,10 +226,10 @@ char	*cp;
 {
 	char	*cp1;
 	char	*cp_start;
-	int	i;
+	int	i, i1;
+	int	len;
 	int	rv;
 	char	buf2[MAX_CMD_LEN];
-	int	i1;
 	char	*filename;
 	FILE	*fp;
 	int	append_flag;
@@ -238,7 +240,8 @@ char	*cp;
 	set_sign_array();
 	cp_start = cp;
 	cp = skip_space(cp);
-	if (*cp == '#') {	/* handle the equation number selector */
+/* handle the equation number selector */
+	if (*cp == '#') {
 		cp++;
 		switch (*cp) {
 		case '+':
@@ -268,8 +271,8 @@ char	*cp;
 		cur_equation = i;
 		return return_result(cur_equation);
 	}
-#if	!SECURE
-	/* handle shell escape */
+#if	(UNIX || CYGWIN) && !SECURE
+/* handle shell escape */
 	if (*cp == '!') {
 		cp1 = getenv("SHELL");
 		if (cp1 == NULL) {
@@ -281,26 +284,24 @@ char	*cp;
 		}
 		cp = skip_space(cp + 1);
 		rv = shell_out(*cp ? cp : cp1);
-		if (rv) {
-			error(_("System command failed."));
-		}
 		return !rv;
 	}
 #endif
-	/* a quick way to get help */
+/* a quick way to get help */
 	if (*cp == '?') {
 		cp = skip_space(cp + 1);
 		input_column += (cp - cp_start);
 		return(help_cmd(cp));
 	}
-/* See if the string pointed to by "cp" is a command. */
+/* See if the string pointed to by cp is a command. */
 /* If so, execute it. */
 	cp1 = cp;
 	while (*cp1 && !isspace(*cp1))
 		cp1++;
+	len = cp1 - cp;	/* length of possible command name in cp */
 	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if (((cp1 - cp) >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, cp1 - cp) == 0)
-		    || (com_list[i].secondary_name && (cp1 - cp) >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
+		if ((len >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, len) == 0)
+		    || (com_list[i].secondary_name && len >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, len) == 0)) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
@@ -336,13 +337,8 @@ char	*cp;
 				gfp = fp;
 			}
 #endif
-			/* remove trailing spaces from the command line */
-		        i1 = strlen(buf2) - 1;
-		        while (i1 >= 0 && isspace(buf2[i1])) {
-        		        buf2[i1] = '\0';
-                		i1--;
-			}
-			/* execute the command */
+			remove_trailing_spaces(buf2);
+/* execute the command by calling the command function */
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
 			if (fp) {	/* if output redirected, close output file */
@@ -356,13 +352,13 @@ char	*cp;
 			return rv;
 		}
 	}
-/* "cp" is not a command, so parse the expression or equation. */
+/* cp is not a command, so parse the expression or equation */
 	i = next_espace();
 	input_column += (cp - cp_start);
 	return process_parse(i, cp);
 }
 
-#if	!SECURE
+#if	(UNIX || CYGWIN) && !SECURE
 /*
  * Execute a system command.  Note that system(3) requires "/bin/sh".
  *
@@ -376,6 +372,7 @@ char	*cp;
 
 	reset_attr();
 	rv = system(cp);
+	printf("\n");
 	if (rv < 0) {
 		perror("system(3) call failed");
 	}
@@ -480,57 +477,59 @@ next_argument:
 			return true;
 		}
 		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
-			P("Commonly Used Geometric Formulas");
-			P("--------------------------------");
+			P("Commonly used standard (Euclidean) geometric formulas");
+			P("-----------------------------------------------------");
 			P("Triangle of base \"b\" and height \"h\":");
-			P("    Area = b*h/2");
+			P("    area = b*h/2");
 			P("Rectangle of length \"l\" and width \"w\":");
-			P("    Area = l*w                    Perimeter = 2*l + 2*w");
+			P("    area = l*w                    perimeter = 2*l + 2*w");
 			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
-			P("    Area = d*(a + b)/2");
+			P("    area = d*(a + b)/2");
 			P("Circle of radius \"r\":");
-			P("    Area = pi*r^2                 Perimeter = 2*pi*r");
-			P("    Area = Perimeter*r/2");
+			P("    area = pi*r^2                 perimeter = 2*pi*r");
+			P("    area = perimeter*r/2");
 			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
-			P("    Volume = l*w*h                Surface area = 2*l*w + 2*l*h + 2*w*h");
+			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h");
 			P("Sphere of radius \"r\":");
-			P("    Volume = 4/3*pi*r^3           Surface area = 4*pi*r^2");
+			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2");
 			P("Right circular cylinder of radius \"r\" and height \"h\":");
-			P("    Volume = pi*r^2*h             Surface area = 2*pi*r*(h + r)");
+			P("    volume = pi*r^2*h             surface_area = 2*pi*r*(h + r)");
 			P("Right circular cone of radius \"r\" and height \"h\":");
-			P("    Volume = pi*r^2*h/3");
-			P("    Base surface area = pi*r^2    Side surface area = pi*r*(r^2 + h^2)^.5\n");
+			P("    volume = pi*r^2*h/3");
+			P("    base_surface_area = pi*r^2    side_surface_area = pi*r*(r^2 + h^2)^.5\n");
 
 			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
-			P("    Sum = (n - 2)*180 degrees     Sum = (n - 2)*pi radians");
+			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
 			return true;
 		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
-			P("Operators have precedence decreasing as indicated:");
+			P("Operators have precedence decreasing as indicated:\n");
+
 			P("    - negate");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
-			P("    * multiply        / divide          % modulus");
+			P("    * multiply        / divide          % modulus         // integral divide");
 			P("    + add             - subtract");
-			P("    = equate");
-			P("Operators in the same precedence level are evaluated left to right.\n");
+			P("    = equate\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Predefined constants and variables follow:");
-			P("    e or e# - the universal constant e (2.7182818284...).");
-			P("    pi or pi# - the universal constant pi (3.1415926535...).");
-			P("    i or i# - the imaginary number (square root of -1).");
+			P("Predefined constants and variables follow:\n");
+
+			P("    e or e# - the universal constant e (2.7182818284...)");
+			P("    pi or pi# - the universal constant pi (3.1415926535...)");
+			P("    i or i# - the imaginary number (square root of -1)");
 			P("The above constants may also be used most anywhere variables are required.");
-			P("    sign, sign1, sign2, sign3, ... - variable that can only be +1 or -1.");
-			P("    integer - variable that may be any integer.");
-			P("    inf - infinity constant (not a variable).\n");
+			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1");
+			P("    inf - floating point infinity constant (not a variable)\n");
 
 			P("Absolute value notation \"|x|\" and dual polarity \"+/-\" are understood.");
 			return true;
 		}
 		if (is_all(cp)) {
-			for (i = 0, row = 1; i < ARR_CNT(com_list); i++) {
+			P("Mathomatic Command Summary");
+			P("--------------------------");
+			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
 				row += display_command(i);
 				if (gfp == stdout && screen_rows && row >= (screen_rows - 5)) {
 					row = 1;
@@ -539,16 +538,18 @@ next_argument:
 					printf("\n");
 				}
 			}
-			P("End of command list.");
+			P("Full documentation can be found at \"http://mathomatic.org/math/doc/\".");
 			return true;
 		}
+		error(_("Unrecognized help topic or command."));
+		return false;
 	}
 	/* default help text: */
+	P("Mathomatic is a small Computer Algebra System (CAS).");
 	P("This help command is provided as a quick reference.");
-	P("Full documentation is available at the website \"www.mathomatic.org\".");
-	P("Type \"help all\" for a summary and syntax of the all commands.");
-	P("Type \"help usage\" for syntax of all commands.");
-	P("Type \"help expressions\" for help with entering expressions.");
+	P("Type \"help equations\" for help with entering expressions and equations.");
+	P("Type \"help all\" for a summary and syntax of all commands.");
+	P("Type \"help usage\" to display the syntax of all commands.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
 	P("\"help\" or \"?\" followed by command names will give info on those commands.\n");
 
@@ -561,7 +562,7 @@ next_argument:
 			fprintf(gfp, " ");
 	}
 
-	P("\n\nTo select an equation in memory, type the equation number at the main prompt.");
+	P("\n\nTo select an equation space, type the equation number at the main prompt.");
 	P("To solve the current equation, type the variable name at the main prompt.");
 	return true;
 }
