@@ -634,7 +634,6 @@ char	*cp;
 		if (!prompt_var(&v))
 			return false;
 	}
-#if	!SILENT
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
@@ -646,10 +645,8 @@ char	*cp;
 		}
 	}
 	if (singularity) {
-		printf(_("Warning: Singularity detected for this expression, result will\n"));
-		printf(_("be wrong if the singularity is within the specified bounds.\n"));
+		warning(_("Singularity detected, result of numerical integration may be wrong."));
 	}
-#endif
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
 		return false;
