@@ -1,12 +1,12 @@
 /*
- * This file contains the C library functions for the Mathomatic library.
+ * This file contains the C functions for the Mathomatic API.
  * Refer to this, if you are going to use the Mathomatic code in other projects.
  *
  * Be sure and define "LIBRARY" when compiling the Mathomatic code for this.
  * And be sure to call "clear_all()" after completing each group of operations,
  * otherwise the equation spaces will fill up.
  *
- * Mathomatic Copyright (C) 1987-2008 George Gesslein II.
+ * Written by George Gesslein II.
  */
 
 #include "../includes.h"
@@ -50,7 +50,7 @@ matho_init()
  * if there is no resulting equation or error message.
  */
 int
-matho_process(char *input, char **outputp)
+matho_process(char *input, const char **outputp)
 {
 	int	i;
 	int	rv;
@@ -90,7 +90,7 @@ matho_process(char *input, char **outputp)
  * Returns true if successful.
  */
 int
-matho_parse(char *input, char **outputp)
+matho_parse(char *input, const char **outputp)
 {
 	int	i;
 	int	rv;
