@@ -1,11 +1,11 @@
 /*
- * This file contains a test for the Mathomatic symbolic math library.
+ * This file contains a test for the Mathomatic symbolic math library and API.
  * Refer to this, if you are going to use the Mathomatic code in other projects.
  *
  * To run this, compile and link with the Mathomatic symbolic math library
  * and create an executable.
  *
- * Mathomatic Copyright (C) 1987-2008 George Gesslein II.
+ * Written by George Gesslein II.
  */
 
 #include <stdio.h>
@@ -15,9 +15,10 @@
 int
 main(int argc, char **argv)
 {
-	char	*cp, *ocp;
-	int	rv;
-	char	buf[1000];
+	char		*cp;		/* character pointer */
+	const char	*ocp;		/* output character pointer */
+	int		rv;		/* return value */
+	char		buf[1000];	/* buffer */
 
 	printf("Mathomatic library test program.\n");
 	if (!matho_init()) {
@@ -34,7 +35,7 @@ main(int argc, char **argv)
 		if (ocp) {
 			printf("%s\n", ocp);
 			if (rv) {
-				free(ocp);
+				free((void *) ocp);
 			}
 		}
 	}
