@@ -15,7 +15,7 @@
 static int	flist();
 static int	flist_recurse();
 
-/* ANSI color code array */
+/* ANSI color code array for 8 color ANSI, we don't use black */
 static int	carray[] = {
 	32,	/* green (default color) */
 	33,	/* yellow */
@@ -30,8 +30,8 @@ static int	carray[] = {
 static char	*html_carray[] = {
 	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
-	"#FF9000",
 	"#FF0000",
+	"#FF9000",
 	"#FF00FF",
 	"#00FFFF",
 	"#0000FF"
@@ -77,11 +77,7 @@ int	color;
 			}
 			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
 		} else {
-			if (bold_colors) {
-				printf("\033[1;%dm", carray[color%ARR_CNT(carray)]);
-			} else {
-				printf("\033[0;%dm", carray[color%ARR_CNT(carray)]);
-			}
+			printf("\033[%d;%dm", bold_colors, carray[color%ARR_CNT(carray)]);
 		}
 		cur_color = color;
 	}
@@ -105,13 +101,13 @@ default_color()
 int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
-int	export_flag;	/* true for exportable format (readable by other math programs) */
+int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
 			/* 1 for Maxima, 2 for other */
 {
 	int	len = 0;
 
 	if (n_lhs[n] <= 0)
-		return len;
+		return 0;
 	if (!export_flag && !high_prec) {
 		len += fprintf(gfp, "#%d: ", n + 1);
 	}
@@ -123,9 +119,9 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-	if (high_prec) {
+	if (high_prec || export_flag) {
 #if	CYGWIN
-		fprintf(gfp, "\r\n");
+		fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
 		fprintf(gfp, "\n");
 #endif
@@ -153,7 +149,7 @@ int	n;	/* equation space number */
 		if (factor_int_flag) {
 			factor_int_sub(n);
 		}
-		return flist_sub(n);
+		return flist_equation(n);
 	} else {
 		/* display in single-line format */
 		if (factor_int_flag) {
@@ -299,15 +295,15 @@ int	lang_code;	/* language code */
 }
 
 /*
- * Display an expression in single line format.
+ * Display an expression in single-line format.
  * Use color if available.
  * The expression is not modified.
  *
  * Return number of characters output (excluding escape sequences).
  */
 int
-list_proc(equation, n, export_flag)
-token_type	*equation;	/* expression pointer */
+list_proc(p1, n, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
 				/* 1 for Maxima, 2 for other */
@@ -318,9 +314,9 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	char	*cp;
 	int	len = 0;
 
-	cur_level = min1 = min_level(equation, n);
+	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
-		j = cur_level - equation[i].level;
+		j = cur_level - p1[i].level;
 		k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -333,30 +329,30 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				len += fprintf(gfp, "(");
 			}
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant < 0.0) {
-					len += fprintf(gfp, "(%.20g)", equation[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += fprintf(gfp, "(%.20g)", p1[i].token.constant);
 				} else {
-					len += fprintf(gfp, "%.20g", equation[i].token.constant);
+					len += fprintf(gfp, "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += fprintf(gfp, "%.*f", finance_option, equation[i].token.constant);
+				len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				len += fprintf(gfp, "%.*g", precision, equation[i].token.constant);
+				len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
 			}
 			break;
 		case VARIABLE:
-			list_var(equation[i].token.variable, 0 - export_flag);
+			list_var(p1[i].token.variable, 0 - export_flag);
 			len += fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
-			switch (equation[i].token.operatr) {
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				break;
@@ -369,11 +365,18 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			case DIVIDE:
 				cp = "/";
 				break;
+			case IDIVIDE:
+				cp = "//";
+				break;
 			case MODULUS:
 				cp = MODULUS_STRING;
 				break;
 			case POWER:
-				cp = "^";
+				if (power_starstar) {
+					cp = "**";
+				} else {
+					cp = "^";
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -396,7 +399,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
  * Store the specified equation in a text string.
  * String should be freed with free() when done.
  *
- * Return string or NULL if can't malloc().
+ * Return string or NULL if error.
  */
 char *
 list_equation(n, export_flag)
@@ -406,6 +409,9 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	int	len;
 	char	*cp;
 
+	if (n_lhs[n] <= 0) {
+		return NULL;
+	}
 	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
@@ -432,25 +438,28 @@ int	export_flag;	/* flag for exportable format (usually false) */
  * Store an expression in a text string.
  * String should be freed with free() when done.
  *
- * Return string or NULL if can't malloc().
+ * Return string or NULL if error.
  */
 char *
-list_expression(equation, n, export_flag)
-token_type	*equation;	/* expression pointer */
+list_expression(p1, n, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;
 {
 	int	len;
 	char	*cp;
 
-	len = list_string_sub(equation, n, NULL, export_flag);
+	if (n <= 0) {
+		return NULL;
+	}
+	len = list_string_sub(p1, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
 		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
-	list_string_sub(equation, n, cp, export_flag);
+	list_string_sub(p1, n, cp, export_flag);
 	return cp;
 }
 
@@ -462,8 +471,8 @@ int		export_flag;
  * Return length (number of characters).
  */
 int
-list_string_sub(equation, n, string, export_flag)
-token_type	*equation;	/* expression pointer */
+list_string_sub(p1, n, string, export_flag)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
@@ -477,9 +486,9 @@ int		export_flag;
 
 	if (string)
 		string[0] = '\0';
-	cur_level = min1 = min_level(equation, n);
+	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
-		j = cur_level - equation[i].level;
+		j = cur_level - p1[i].level;
 		k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
@@ -490,31 +499,31 @@ int		export_flag;
 				APPEND("(");
 			}
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant < 0.0) {
-					snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					snprintf(buf, sizeof(buf), "(%.20g)", p1[i].token.constant);
 				} else {
-					snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
+					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.*f", finance_option, equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 			}
 			APPEND(buf);
 			break;
 		case VARIABLE:
-			list_var(equation[i].token.variable, 0 - export_flag);
+			list_var(p1[i].token.variable, 0 - export_flag);
 			APPEND(var_str);
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
-			switch (equation[i].token.operatr) {
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				break;
@@ -527,11 +536,18 @@ int		export_flag;
 			case DIVIDE:
 				cp = "/";
 				break;
+			case IDIVIDE:
+				cp = "//";
+				break;
 			case MODULUS:
 				cp = MODULUS_STRING;
 				break;
 			case POWER:
-				cp = "^";
+				if (power_starstar) {
+					cp = "**";
+				} else {
+					cp = "^";
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -553,15 +569,15 @@ int		export_flag;
  * list_code().
  */
 int
-int_expr(equation, n)
-token_type	*equation;	/* expression pointer */
+int_expr(p1, n)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
 
 	for (i = 0; i < n; i++) {
-		if ((equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
-		    || (equation[i].kind == VARIABLE && equation[i].token.variable <= IMAGINARY)) {
+		if ((p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
+		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
 			return false;
 		}
 	}
@@ -570,24 +586,29 @@ int		n;		/* length of expression */
 
 /*
  * Display an equation as C, Java, or Python code.
+ *
+ * Return length of output (number of characters).
  */
-void
+int
 list_c_equation(en, language, int_flag)
 int	en;		/* equation space number */
 int	language;
 int	int_flag;	/* integer arithmetic flag */
 {
+	int	len = 0;
+
 	if (n_lhs[en] <= 0)
-		return;
-	list_code(lhs[en], &n_lhs[en], language, int_flag);
+		return 0;
+	len += list_code(lhs[en], &n_lhs[en], language, int_flag);
 	if (n_rhs[en]) {
-		fprintf(gfp, " = ");
-		list_code(rhs[en], &n_rhs[en], language, int_flag);
+		len += fprintf(gfp, " = ");
+		len += list_code(rhs[en], &n_rhs[en], language, int_flag);
 	}
 	if (language != 2) {
-		fprintf(gfp, ";");
+		len += fprintf(gfp, ";");
 	}
 	fprintf(gfp, "\n");
+	return len;
 }
 
 /*
@@ -598,8 +619,8 @@ int	int_flag;	/* integer arithmetic flag */
  */
 int
 list_code(equation, np, language, int_flag)
-token_type	*equation;	/* expression pointer */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 int		language;	/* if 0, generate C code; if 1, generate Java code; if 2, generate Python code */
 int		int_flag;	/* integer arithmetic flag, should work with any language */
 {
@@ -691,10 +712,11 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = " - ";
 				break;
 			case TIMES:
-				cp = " * ";
+				cp = "*";
 				break;
 			case DIVIDE:
-				cp = " / ";
+			case IDIVIDE:
+				cp = "/";
 				break;
 			case MODULUS:
 				cp = MODULUS_STRING;
@@ -729,7 +751,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
  * or zero on failure.
  */
 int
-flist_sub(n)
+flist_equation(n)
 int	n;	/* equation space number */
 {
 	int	sind;
@@ -814,8 +836,8 @@ make_smaller:
  * Return the width of the expression (that is, the required number of screen columns).
  */
 static int
-flist(equation, n, out_flag, pos, highp, lowp)
-token_type	*equation;	/* expression pointer */
+flist(p1, n, out_flag, pos, highp, lowp)
+token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		out_flag;
 int		pos;
@@ -823,7 +845,7 @@ int		*highp, *lowp;
 {
 	int	i;
 
-	i = flist_recurse(equation, n, out_flag, 0, pos, 1, highp, lowp);
+	i = flist_recurse(p1, n, out_flag, 0, pos, 1, highp, lowp);
 	if (out_flag) {
 		default_color();
 	}
@@ -831,8 +853,8 @@ int		*highp, *lowp;
 }
 
 static int
-flist_recurse(equation, n, out_flag, line, pos, cur_level, highp, lowp)
-token_type	*equation;
+flist_recurse(p1, n, out_flag, line, pos, cur_level, highp, lowp)
+token_type	*p1;
 int		n;
 int		out_flag;
 int		line;
@@ -872,10 +894,10 @@ check_again:
 	stop_at = n;
 	div_loc = -1;
 	for (i = ii; i < n; i++) {
-		if (equation[i].kind == OPERATOR && equation[i].token.operatr == DIVIDE) {
-			level = equation[i].level;
+		if (p1[i].kind == OPERATOR && p1[i].token.operatr == DIVIDE) {
+			level = p1[i].level;
 			for (j = i - 2; j > 0; j -= 2) {
-				if (equation[j].level < level)
+				if (p1[j].level < level)
 					break;
 			}
 			j++;
@@ -887,7 +909,7 @@ check_again:
 					div_loc = i;
 					stop_at = j;
 				} else if (j == stop_at) {
-					if (level < equation[div_loc].level)
+					if (level < p1[div_loc].level)
 						div_loc = i;
 				}
 			}
@@ -895,10 +917,10 @@ check_again:
 	}
 	for (i = ii; i < n; i++) {
 		if (i == stop_at) {
-			j = cur_level - equation[div_loc].level;
+			j = cur_level - p1[div_loc].level;
 			k = abs(j) - 1;
 		} else {
-			j = cur_level - equation[i].level;
+			j = cur_level - p1[i].level;
 			k = abs(j);
 		}
 		for (i1 = 1; i1 <= k; i1++) {
@@ -919,14 +941,14 @@ check_again:
 			}
 		}
 		if (i == stop_at) {
-			level = equation[div_loc].level;
-			len1 = flist_recurse(&equation[stop_at], div_loc - stop_at, false, line + 1, pos + len, level, &high, &low);
+			level = p1[div_loc].level;
+			len1 = flist_recurse(&p1[stop_at], div_loc - stop_at, false, line + 1, pos + len, level, &high, &low);
 			l1 = (2 * (line + 1)) - low;
 			for (j = div_loc + 2; j < n; j += 2) {
-				if (equation[j].level <= level)
+				if (p1[j].level <= level)
 					break;
 			}
-			len2 = flist_recurse(&equation[div_loc+1], j - (div_loc + 1), false, line - 1, pos + len, level, &high, &low);
+			len2 = flist_recurse(&p1[div_loc+1], j - (div_loc + 1), false, line - 1, pos + len, level, &high, &low);
 			l2 = (2 * (line - 1)) - high;
 			ii = j;
 			len_div = max(len1, len2);
@@ -934,7 +956,7 @@ check_again:
 			if (len1 < len_div) {
 				j = (len_div - len1) / 2;
 			}
-			flist_recurse(&equation[stop_at], div_loc - stop_at, out_flag, l1, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[stop_at], div_loc - stop_at, out_flag, l1, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
@@ -954,7 +976,7 @@ check_again:
 			if (len2 < len_div) {
 				j = (len_div - len2) / 2;
 			}
-			flist_recurse(&equation[div_loc+1], ii - (div_loc + 1), out_flag, l2, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[div_loc+1], ii - (div_loc + 1), out_flag, l2, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
@@ -962,27 +984,27 @@ check_again:
 			len += len_div;
 			goto check_again;
 		}
-		switch (equation[i].kind) {
+		switch (p1[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0) {
-				equation[i].token.constant = 0.0; /* fix -0 */
+			if (p1[i].token.constant == 0.0) {
+				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, 0);
+			len += list_var(p1[i].token.variable, 0);
 			if (oflag)
 				fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
 			/* len must be incremented here by the number of columns the display of the operator takes */
-			switch (equation[i].token.operatr) {
+			switch (p1[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
 				len += 3;
@@ -1007,13 +1029,22 @@ check_again:
 				cp = "/";
 				len++;
 				break;
+			case IDIVIDE:
+				cp = "//";
+				len += 2;
+				break;
 			case MODULUS:
 				cp = MODULUS_STRING;
 				len += strlen(cp);
 				break;
 			case POWER:
-				cp = "^";
-				len++;
+				if (power_starstar) {
+					cp = "**";
+					len += 2;
+				} else {
+					cp = "^";
+					len++;
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
