@@ -1,6 +1,6 @@
 /*
- * This file contains main() and the startup code for Mathomatic.
- * Leave this file out when compiling for the symbolic math library.
+ * This file contains main() and the startup code for Mathomatic,
+ * which is an algebraic manipulator written entirely in C.
  *
  * Copyright (C) 1987-2008 George Gesslein II.
  *
@@ -35,22 +35,29 @@
 
 #if	!LIBRARY
 #include "includes.h"
+#if	UNIX || CYGWIN
 #include <sys/ioctl.h>
 #include <termios.h>
+#endif
+#if	TRAP_FPERRORS	/* Do not use this define!  Currently does not work. */
+#include <fenv.h>
+#endif
 
 static void usage();
 static int set_signals();
 
+static char	rc_file[MAX_CMD_LEN];
+
 int
 main(argc, argv)
 int	argc;
 char	**argv;
 {
-	extern char	*optarg;
+	extern char	*optarg;	/* set by getopt(3) */
 	extern int	optind;
 
 	int		i;
-	char		*cp;
+	char		*cp = NULL;
 	double		numerator, denominator;
 	double		multiplier;
 	int		coption = false;
@@ -72,14 +79,13 @@ char	**argv;
 			coption++;
 			break;
 		case 'h':
-			readline_enabled = false;
 			html_flag = true;
 			break;
 		case 'm':
 			multiplier = strtod(optarg, &cp) * DEFAULT_N_TOKENS;
-			if (*cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
+			if (cp == NULL || *cp || multiplier < 100 || multiplier >= (INT_MAX / 3)) {
 				fprintf(stderr, _("%s: Invalid memory size multiplier specified.\n"), prog_name);
-				exit(1);
+				exit(2);
 			}
 			n_tokens = (int) multiplier;
 			break;
@@ -106,8 +112,10 @@ char	**argv;
 	if (test_mode || html_flag) {
 		screen_columns = 0;
 		screen_rows = 0;
+#if	UNIX || CYGWIN
 	} else {
 		get_screen_size();
+#endif
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
@@ -122,24 +130,23 @@ char	**argv;
 #endif
 		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
 		printf(_("Copyright (C) 1987-2008 George Gesslein II.\n"));
-		printf(_("This is free software; see the source for copying conditions.  There is NO\n"));
-		printf(_("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"));
+		printf(_("This is free software with NO WARRANTY.\n\n"));
+
 		printf(_("%d equation spaces available, %lld kilobytes per equation space.\n"),
 		    N_EQUATIONS, (long long) n_tokens * sizeof(token_type) * 2LL / 1000LL);
 	}
 #if	!SECURE
 	/* read the user options initialization file */
 	if (!test_mode && !load_rc()) {
-		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, RC_FILE);
+		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, rc_file);
 	}
 #endif
 	if (test_mode) {
 		color_flag = false;
-	}
-	if (coption & 1) {
+	} else if (coption & 1) {
 		color_flag = !color_flag;
 	}
-	if (!test_mode && !quiet_mode && color_flag) {
+	if (!quiet_mode && color_flag) {
 		printf(_("%s color mode enabled.\n"), html_flag ? "HTML" : "ANSI");
 	}
 	if ((i = setjmp(jmp_save)) != 0) {
@@ -179,6 +186,7 @@ char	**argv;
 		process(cp);
 	}
 	exit_program(0);
+	return(0);	/* so the compiler doesn't complain */
 }
 
 /*
@@ -187,18 +195,18 @@ char	**argv;
 static void
 usage()
 {
-	fprintf(stderr, _("Mathomatic version %s - automatic algebraic manipulator\n\n"), VERSION);
-	fprintf(stderr, _("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
-	fprintf(stderr, _("Options:\n"));
-	fprintf(stderr, _("  -c            Toggle color mode.\n"));
-	fprintf(stderr, _("  -h            Enable HTML output mode and disable readline.\n"));
-	fprintf(stderr, _("  -m number     Specify a memory size multiplier.\n"));
-	fprintf(stderr, _("  -q            Set quiet mode (don't display prompts).\n"));
-	fprintf(stderr, _("  -r            Disable readline.\n"));
-	fprintf(stderr, _("  -t            Set test mode.\n"));
-	fprintf(stderr, _("  -u            Set unbuffered output.\n"));
-	fprintf(stderr, _("  -v            Display version number, compile flags, and memory usage.\n"));
-	exit(1);
+	printf("Mathomatic version %s - automatic algebraic manipulator\n\n", VERSION);
+	printf("Usage: %s [ options ] [ input_files ]\n\n", prog_name);
+	printf("Options:\n");
+	printf("  -c            Toggle color mode.\n");
+	printf("  -h            Enable HTML output mode.\n");
+	printf("  -m number     Specify a memory size multiplier.\n");
+	printf("  -q            Set quiet mode (don't display prompts).\n");
+	printf("  -r            Disable readline.\n");
+	printf("  -t            Set test mode.\n");
+	printf("  -u            Set unbuffered output.\n");
+	printf("  -v            Display version number, compile flags, and memory usage.\n");
+	exit(2);
 }
 
 /*
@@ -213,10 +221,16 @@ set_signals()
 
 	if (signal(SIGFPE, fphandler) == SIG_ERR)
 		rv = false;
+#if	TRAP_FPERRORS
+	if (feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW) < 0)
+		rv = false;
+#endif
 	if (signal(SIGINT, inthandler) == SIG_ERR)
 		rv = false;
+#if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
 		rv = false;
+#endif
 #if	TIMEOUT_SECONDS
 	if (signal(SIGALRM, alarmhandler) == SIG_ERR)
 		rv = false;
@@ -257,26 +271,26 @@ char	*cp;
 int
 load_rc()
 {
-	FILE	*fp;
+	FILE	*fp = NULL;
 	char	buf[MAX_CMD_LEN];
 	char	*cp;
 	int	rv = true;
 
+	cp = getenv("HOME");
+	if (cp) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, ".mathomaticrc");
+		fp = fopen(rc_file, "r");
+	}
 #if	CYGWIN
-	snprintf(buf, sizeof(buf), "%s/%s", dir_path, RC_FILE);
-	fp = fopen(buf, "r");
 	if (fp == NULL) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", dir_path, "mathomatic.rc");
+		fp = fopen(rc_file, "r");
+	}
+#endif
+	if (fp == NULL) {
+		rc_file[0] = '\0';
 		return true;
 	}
-#else
-	cp = getenv("HOME");
-	if (cp == NULL)
-		return true;
-	snprintf(buf, sizeof(buf), "%s/%s", cp, RC_FILE);
-	fp = fopen(buf, "r");
-	if (fp == NULL)
-		return true;
-#endif
 	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
 		set_error_level(cp);
 		if (!set_options(cp))
@@ -289,25 +303,47 @@ load_rc()
 
 /*
  * Floating point exception handler.
- * Usually doesn't work in most operating systems.
+ * I do not know the proper function calls that should be here to make the GNU TRAP_FPERRORS define work.
+ * Currently it goes in an endless loop when an FP exception occurs and TRAP_FPERRORS is true,
+ * because when this function returns, the exception immediately happens again.
+ * Therefore, do not define TRAP_FPERRORS!
  */
 void
 fphandler(sig)
 int	sig;
 {
-/*	error(_("Floating point exception.")); */
+#if	TRAP_FPERRORS
+	warning(_("Floating point exception."));
+/*
+	feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW);
+	signal(SIGFPE, fphandler);
+*/
+#endif
 }
 
 /*
- * Control-C (interrupt) handler.
- * Abruptly quit this program.
+ * Control-C (interrupt) signal handler.
+ * Interrupts processing and returns to main prompt through a polling mechanism.
+ * If it can't, repeated calls terminate this program.
  */
 void
 inthandler(sig)
 int	sig;
 {
-	printf(_("\nControl-C pressed, quitting...\n"));
-	exit_program(1);
+	abort_flag++;
+	switch (abort_flag) {
+	case 0:
+	case 1:
+		/* wait for graceful abort */
+		return;
+	case 2:
+		printf(_("\nPress Control-C once more to quit program.\n"));
+		break;
+	default:
+		/* abruptly quit this program */
+		printf(_("\nControl-C repeatedly pressed, quitting...\n"));
+		exit_program(1);
+	}
 }
 
 /*
@@ -321,8 +357,9 @@ int	sig;
 	exit_program(1);
 }
 
+#if	UNIX || CYGWIN
 /*
- * Get the screen (window) width and height.
+ * Get the screen (window) width and height from the operating system.
  */
 void
 get_screen_size()
@@ -342,7 +379,7 @@ get_screen_size()
 }
 
 /*
- * Window resize handler.
+ * Window resize signal handler.
  */
 void
 resizehandler(sig)
@@ -350,6 +387,7 @@ int	sig;
 {
 	get_screen_size();
 }
+#endif
 
 /*
  * Exit this program and return to the Operating System.
