@@ -1,7 +1,7 @@
 /*
  * Mathomatic simplifying and polynomial routines.
  * Includes polynomial and smart division, polynomial factoring, etc.
- * tlhs and trhs are wiped out by most of these routines.
+ * Globals tlhs[] and trhs[] are wiped out by most of these routines.
  *
  * The polynomial division and GCD routines here are not recursive, due to static
  * storage areas.  This limits the polynomial GCD to univariate operation.
@@ -21,13 +21,8 @@ int		n_quotient;			/* length of expression in quotient[] */
 token_type	gcd_divisor[DIVISOR_SIZE];	/* static data area for polynomial GCD routine */
 int		len_d;				/* length of expression in gcd_divisor[] */
 
-static int	do_gcd();
-static int	pf_recurse();
-static int	pf_sub();
-static int	mod_recurse();
-static int	polydiv_recurse();
-static int	pdiv_recurse();
-static int	poly_div_sub();
+static int	do_gcd(), pf_recurse(), pf_sub(), mod_recurse(), polydiv_recurse(), pdiv_recurse(), poly_div_sub();
+static int	find_highest_count();
 static void	save_factors();
 
 /*
@@ -44,7 +39,7 @@ sort_type	*p1, *p2;
  * Return true if passed expression is strictly a single polynomial term in variable v.
  * The exponent may be anything, as long as it doesn't contain the variable.
  */
-static inline int
+static int
 poly_in_v_sub(p1, n, v, allow_divides)
 token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
@@ -122,7 +117,7 @@ int		allow_divides;	/* allow variable to be right of a divide (negative exponent
 }
 
 /*
- * Factor polynomials.
+ * Factor polynomials by calling pf_sub() for every additive subexpression.
  * Factors repeated factor polynomials (like (x+1)^5)
  * and symbolic factor polynomials (like (x+a)*(x+b)).
  *
@@ -205,8 +200,11 @@ int		*np, loc, len, level;
 	if (v == 0)
 		return false;
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if (setjmp(jmp_save) != 0) {
+	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (i == 13) {	/* critical error code */
+			longjmp(jmp_save, i);
+		}
 		return(modified || symbolic_modified);
 	}
 /* First factor polynomials with repeated factors */
@@ -418,9 +416,9 @@ int		*np, loc1, len, level;
 }
 
 /*
- * Remove level 1 trivial factors and divides from "tlhs".
+ * Remove level 1 trivial factors and divides from tlhs.
  *
- * Return true if result is a level 1 plus type expression.
+ * Return true if result is a level 1 additive expression.
  * If this returns false, nothing is removed.
  */
 int
@@ -499,7 +497,7 @@ long		*vp;
 	int	i;
 	int	count;
 
-	for (count = 1;; count++) {
+	for (count = 1; count < 50; count++) {
 		switch (poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
 		case 0:
 			/* divide failed */
@@ -518,14 +516,15 @@ long		*vp;
 		n_trhs = len_d;
 		len_d = i;
 	}
+	return 0;
 }
 
 /*
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
  *
  * Return true if successful.
- * Return the GCD in "trhs".
- * Return "larger"/GCD in "tlhs".
+ * Return the GCD in trhs[].
+ * Return larger/GCD in tlhs[].
  */
 int
 poly_gcd(larger, llen, smaller, slen, v)
@@ -563,7 +562,7 @@ long		v;
 		blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 		len_d = n_tlhs;
 		if (poly_div(larger, llen, gcd_divisor, len_d, &v) != 2) {
-			debug_string(0, "Polynomial GCD found, but larger divide failed.");
+			warning("Polynomial GCD found, but larger divide failed in poly_gcd().");
 			return 0;
 		}
 	}
@@ -581,8 +580,8 @@ long		v;
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
  *
  * Return true if successful.
- * Return "larger"/GCD in "tlhs".
- * Return "smaller"/GCD in "trhs".
+ * Return larger/GCD in tlhs[].
+ * Return smaller/GCD in trhs[].
  */
 int
 poly2_gcd(larger, llen, smaller, slen, v)
@@ -591,6 +590,7 @@ token_type	*smaller;	/* smaller polynomial */
 int		llen, slen;
 long		v;
 {
+	int		i;
 	jmp_buf		save_save;
 	int		count;
 
@@ -603,13 +603,18 @@ long		v;
 	n_trhs = llen;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
+#if	true
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if (setjmp(jmp_save) != 0) {
+	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (i == 13) {	/* critical error code */
+			longjmp(jmp_save, i);
+		}
 		return 0;
 	}
 	uf_simp(tlhs, &n_tlhs);
 	blt(jmp_save, save_save, sizeof(jmp_save));
+#endif
 	if (!level1_plus_count(tlhs, n_tlhs))
 		return 0;
 	if (n_tlhs > ARR_CNT(gcd_divisor))
@@ -623,7 +628,7 @@ long		v;
 		if (!level1_plus_count(gcd_divisor, len_d))
 			return 0;
 		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
-			debug_string(0, "Polynomial GCD found, but smaller divide failed.");
+			warning("Polynomial GCD found, but smaller divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -635,7 +640,7 @@ long		v;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		if (poly_div(larger, llen, tlhs, n_tlhs, &v) != 2) {
-			debug_string(0, "Polynomial GCD found, but larger divide failed.");
+			warning("Polynomial GCD found, but larger divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -1133,10 +1138,9 @@ next_thingy:
  * Do polynomial division.
  *
  * Returns non-zero if successful, 2 if remainder is 0.
- * Quotient is returned in "tlhs" and remainder in "trhs".
+ * Quotient is returned in tlhs[] and remainder in trhs[].
  *
- * If "*vp" is 0, automatically select the best base variable,
- * and return it in "*vp".
+ * If *vp is 0, automatically select the best base variable and return it in *vp.
  */
 int
 poly_div(d1, len1, d2, len2, vp)
@@ -1146,16 +1150,19 @@ token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 long		*vp;		/* variable pointer to base variable */
 {
-	int		j;
+	int		i, j;
 	int		old_partial;
 	jmp_buf		save_save;
 
 	old_partial = partial_flag;
 	partial_flag = false;	/* We want full unfactoring. */
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if (setjmp(jmp_save) != 0) {
+	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		partial_flag = old_partial;
+		if (i == 13) {	/* critical error code */
+			longjmp(jmp_save, i);
+		}
 		return false;
 	}
 	j = poly_div_sub(d1, len1, d2, len2, vp);
@@ -1165,7 +1172,7 @@ long		*vp;		/* variable pointer to base variable */
 }
 
 /*
- * Do the actual polynomial division.
+ * Do the actual polynomial division using a simple generalized polynomial long division algorithm.
  */
 static int
 poly_div_sub(d1, len1, d2, len2, vp)
@@ -1207,7 +1214,7 @@ long		*vp;
 	/* Display debugging info. */
 	if (debug_level >= 3) {
 		list_var(*vp, 0);
-		printf(_("poly_div() starts using variable (%s):\n"), var_str);
+		printf(_("poly_div() starts using base variable (%s):\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
@@ -1351,7 +1358,7 @@ long		*vp;
  * of variables, not including any terms with no variables.
  *
  * Returns true if successful.
- * Quotient is returned in "tlhs" and remainder in "trhs".
+ * Quotient is returned in tlhs[] and remainder in trhs[].
  */
 int
 smart_div(d1, len1, d2, len2)
@@ -1650,13 +1657,11 @@ int		*tp1, *lentp1;
 }
 
 /*
- * This routine automatically finds the best normal variable to do
- * polynomial division with.
+ * This routine automatically finds the best normal variable to do polynomial division with.
  *
- * Returns 0 if nothing was found, otherwise variable is returned
- * in "*vp1".
+ * Returns 0 if nothing was found, otherwise the selected base variable is returned in *vp1.
  */
-int
+static int
 find_highest_count(p1, n1, p2, n2, vp1)
 token_type	*p1;		/* pointer to dividend expression */
 int		n1;		/* length of dividend */
