@@ -1,7 +1,5 @@
 /*
- * Long integer square root function.
- *
- * Copyright (C) 2007 George Gesslein II.
+ * Tested long integer square root functions.
  */
 
 #include <stdio.h>
@@ -9,9 +7,21 @@
 #define	true	1
 #define	false	0
 
+/*
+ * Return true if x is the truncated integer square root of y.
+ */
+int
+verify_lsqrt(long y, long x)
+{
+	if ((long long) x * x > y || ((long long) x + 1) * ((long long) x + 1) <= y) {
+		return false;
+	}
+	return true;
+}
+
 #if	false
 /*
- * Return the truncated integer square root using the Babylonian iterative approximation method.
+ * Returns the truncated integer square root of y using the Babylonian iterative approximation method.
  * Return -1 on error.
  */
 long
@@ -45,23 +55,21 @@ lsqrt(long y)
 			break;
 		x_old = x_new;
 	}
-/* optional test code to make sure that the answer is right: */
-	if ((long long) x_old * x_old > y || ((long long) x_old + 1) * ((long long) x_old + 1) <= y) {
-		fprintf(stderr, "Error in %s().\n", __func__);
-		return -1L;
-	}
 	return x_old;
 }
 #else
 /*
  * Faster and simpler binary digit algorithm for calculating the truncated integer square root.
- * Returns -1 on error.
+ * Copied from Wikipedia and modified.
+ *
+ * Returns the truncated integer square root of num.
+ * Return -1 on error.
  */
 long
 lsqrt(long num)
 {
 	long op = num;
-	long res = 0;
+	long result = 0;
 	long one = 1L << (sizeof(num) * 8 - 2); // The second-to-top bit is set
 
 	if (num <= 0) {
@@ -76,13 +84,19 @@ lsqrt(long num)
 		one >>= 2;
 
 	while (one != 0) {
-		if (op >= res + one) {
-			op -= res + one;
-			res += 2 * one;
+		if (op >= result + one) {
+			op -= result + one;
+			result += 2 * one;
 		}
-		res >>= 1;
+		result >>= 1;
 		one >>= 2;
 	}
-	return res;
+/*
+	if (!verify_lsqrt(num, result)) {
+		fprintf(stderr, "Error in %s().\n", __func__);
+		return -1L;
+	}
+*/
+	return result;
 }
 #endif
