@@ -19,8 +19,9 @@
 #define	true	1
 #define	false	0
 
-#define	MAX_LINES	100	/* max number of lines of Pascal's triangle allowed */
+#define	MAX_LINES	1000	/* max number of lines of Pascal's triangle allowed */
 
+void	allocate_triangle(void);
 void	calculate_triangle(void);
 void	display_triangle(void);
 int	center_buf(int line_number, int cell_size);
@@ -38,7 +39,6 @@ char		*prog_name = "matho-pascal";
 int
 main(int argc, char *argv[])
 {
-	int		i;
 	struct winsize	ws;
 
         prog_name = strdup(basename(argv[0]));
@@ -68,18 +68,26 @@ main(int argc, char *argv[])
 	}
 	if (lines <= 0 || lines > MAX_LINES) {
 		fprintf(stderr, "%s: Number of lines out of range (1..%d).\n", prog_name, MAX_LINES);
-		exit(1);
+		exit(2);
 	}
+	allocate_triangle();
+	calculate_triangle();
+	display_triangle();
+	exit(0);
+}
+
+void
+allocate_triangle()
+{
+	int	i;
+
 	for (i = 0; i < lines; i++) {
-		array[i] = (long double *) malloc(sizeof(long double) * lines);
+		array[i] = (long double *) malloc(sizeof(long double) * (i + 1));
 		if (array[i] == NULL) {
 			fprintf(stderr, "%s: Not enough memory.\n", prog_name);
 			exit(2);
 		}
 	}
-	calculate_triangle();
-	display_triangle();
-	exit(0);
 }
 
 void
@@ -90,7 +98,7 @@ calculate_triangle()
 	for (i = 0; i < lines; i++) {
 		for (j = 0; j <= i; j++) {
 			if (j == 0 || j == i) {
-				array[i][j] = 1.0;
+				array[i][j] = 1.0;	/* the line begins and ends with 1 */
 			} else {
 				array[i][j] = array[i-1][j-1] + array[i-1][j];
 			}
@@ -166,7 +174,7 @@ center_buf(int line_number, int cell_size)
 void
 usage()
 {
-	fprintf(stderr, "Usage: %s [number-of-lines]\n\n", prog_name);
-	fprintf(stderr, "Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
-	exit(1);
+	printf("Usage: %s [number-of-lines]\n\n", prog_name);
+	printf("Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
+	exit(2);
 }
