@@ -9,7 +9,7 @@
  *
  * This program finds the minimum number of positive integers squared
  * and summed to equal a given positive integer.  If the number of squared
- * integers is 2, all combinations with 2 squares are displayed,
+ * integers is 2 (default "multi"), all combinations with 2 squares are displayed,
  * otherwise only the first solution found is displayed.
  *
  * If nothing is specified on the command line, the program starts at 0 and
@@ -22,12 +22,15 @@
 #include <stdlib.h>
 #include <math.h>
 #include <errno.h>
+#include <assert.h>
 
 #define	true	1
 #define	false	0
 
 #define	squared(a)	((a) * (a))
 
+int	multi = 2;	/* display all combinations for this number of squares or less */
+
 long	squares[4];
 
 long	lsqrt(long y);
@@ -52,9 +55,7 @@ try_again:
 			save = d2;
 		}
 		squares[i] = lsqrt(d2);
-		if (squares[i] < 0) {
-			exit(1);
-		}
+		assert(squares[i] >= 0);
 		d2 -= squared(squares[i]);
 		i++;
 		if (i >= n) {
@@ -82,11 +83,8 @@ try_again:
 		}
 		printf("\n");
 skip_print:
-		if (!found_one) {
-			fprintf(stderr, "Found bug!\n");
-			exit(1);
-		}
-		if (n != 2) {
+		assert(found_one);
+		if (n < 2 || n > multi) {
 			return found_one;
 		}
 	}
@@ -117,9 +115,9 @@ skip_print:
 }
 
 /*
- * Display the shortest sum of the squares for long integer "d1".
+ * Display the shortest sums of squares for long integer "d1".
  *
- * Return number of squares.
+ * Return the minimum number of summed squares required to represent "d1".
  */
 int
 findsq(long d1)
@@ -130,11 +128,10 @@ findsq(long d1)
 		return 2;
 	if (sumsq(d1, 3))
 		return 3;
-	if (!sumsq(d1, 4)) {
-		fprintf(stderr, "Whoops!  Can't find the sum of four squares that equal %ld.\n", d1);
-		exit(1);
-	}
-	return 4;
+	if (sumsq(d1, 4))
+		return 4;
+	fprintf(stderr, "Whoops!  Can't find the sum of four squares that equal %ld.\n", d1);
+	exit(1);
 }
 
 int
@@ -142,12 +139,11 @@ main(int argc, char *argv[])
 {
 	int	i;
 	long	d1 = 0;
-	char	*cp;
 
 	if (argc > 1) {
 		for (i = 1; i < argc; i++) {
 			errno = 0;
-			d1 = strtol(argv[i], &cp, 10);
+			d1 = strtol(argv[i], NULL, 10);
 			if (errno || d1 <= 0) {
 				fprintf(stderr, "Invalid number.\n");
 				exit(1);
