@@ -18,505 +18,13 @@ static int	const_recurse();
 static int	compare_recurse();
 static int	order_recurse();
 
-/*
- * Quickly and very basically simplify an equation space.
- * No factoring is done.
- */
-void
-simp_sub(n)
-int	n;	/* equation space number to simplify */
-{
-	if (n_lhs[n] <= 0)
-		return;
-	simp_loop(lhs[n], &n_lhs[n]);
-	if (n_rhs[n]) {
-		simp_loop(rhs[n], &n_rhs[n]);
-	}
-}
-
-/*
- * For quick, mid-range simplification of an expression.
- */
-void
-simp_side(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-{
-	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
-}
-
-/*
- * This function is the mid-range simplifier used by the solver.
- */
-void
-simps_side(equation, np, zsolve)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-int		zsolve;		/* true for solving for zero */
-{
-	elim_loop(equation, np);
-	simp_constant_power(equation, np);
-	do {
-		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
-	} while (super_factor(equation, np, 0));
-}
-
-/*
- * This function is used by the factor command.
- */
-void
-simpv_side(equation, np, v)
-token_type	*equation;	/* pointer to the beginning of equation side to factor */
-int		*np;		/* pointer to length of equation side */
-long		v;		/* variable to factor, 0 for all variables */
-{
-	if (*np == 0)
-		return;
-	simp_ssub(equation, np, v, 0.0, v == 0, true, 2);
-}
-
-/*
- * Convert expression with any algebraic fractions into a single fraction.
- */
-void
-frac_side(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side */
-int		*np;		/* pointer to length of equation side */
-{
-	if (*np == 0)
-		return;
-	do {
-		simpb_side(equation, np, false, 2);
-	} while (super_factor(equation, np, 2));
-}
-
-/*
- * Simplify and approximate for the calculate command.
- * Includes complex number simplification.
- */
-void
-calc_simp(equation, np)
-token_type	*equation;
-int		*np;
-{
-	approximate_roots = true;
-	subst_constants(equation, np);
-	simp_side(equation, np);
-	uf_power(equation, np);
-	factorv(equation, np, IMAGINARY);
-	ufactor(equation, np);
-	factorv(equation, np, IMAGINARY);
-	uf_simp(equation, np);
-	factorv(equation, np, IMAGINARY);
-	simp_side(equation, np);
-	uf_simp(equation, np);
-	approximate_roots = false;
-}
-
-/*
- * Try to eliminate imaginary numbers (i#) from an equation side by converting
- * expressions like (i#*(b^.5)) to ((-1*b)^.5).
- */
-void
-simp_i(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	i;
-	int	level;
-
-	simp_loop(equation, np);
-	for (i = 0; i < *np; i += 2) {
-		if (equation[i].kind == VARIABLE && equation[i].token.variable == IMAGINARY) {
-			if (*np + 2 > n_tokens) {
-				error_huge();
-			}
-			level = equation[i].level + 1;
-			blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
-			*np += 2;
-			equation[i].level = level;
-			equation[i].kind = CONSTANT;
-			equation[i].token.constant = -1.0;
-			i++;
-			equation[i].level = level;
-			equation[i].kind = OPERATOR;
-			equation[i].token.operatr = POWER;
-			i++;
-			equation[i].level = level;
-			equation[i].kind = CONSTANT;
-			equation[i].token.constant = 0.5;
-		}
-	}
-	do {
-		do {
-			do {
-				do {
-					organize(equation, np);
-					combine_constants(equation, np, false);
-				} while (elim_k(equation, np));
-			} while (simp_pp(equation, np));
-		} while (factor_power(equation, np));
-	} while (factor_times(equation, np));
-	simp_loop(equation, np);
-}
-
-/*
- * Main simplify code used by the simplify command.
- * This is the slowest and most thorough simplify of all.
- * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
- * then does generalized polynomial simplifications.
- *
- * tlhs and trhs are wiped out.
- */
-void
-simpa_side(equation, np, quick_flag, frac_flag)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of the equation side */
-int		quick_flag;	/* "simplify quick" option */
-int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxima's ratsimp() */
-{
-	int		flag, poly_flag = true;
-	jmp_buf		save_save;
-
-	if (*np <= 1)	/* no need to simplify a single constant or variable */
-		return;
-	simp_loop(equation, np);
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 1);
-	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
-	rationalize(equation, np);
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	simp_loop(equation, np);
-	/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
-	uf_repeat(equation, np);
-	do {
-		elim_loop(equation, np);
-	} while (mod_simp(equation, np));
-	/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
-	simp_i(equation, np);
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np);
-	uf_neg_help(equation, np);
-	uf_times(equation, np);
-	do {
-		do {
-			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-			} while (poly_gcd_simp(equation, np));
-		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
-	unsimp_power(equation, np);
-	uf_times(equation, np);
-	factorv(equation, np, IMAGINARY);
-	uf_pplus(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
-	uf_times(equation, np);
-	uf_pplus(equation, np);
-	factorv(equation, np, IMAGINARY);
-	uf_power(equation, np);
-	do {
-		do {
-			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
-	if (quick_flag) {
-do_quick:
-		uf_tsimp(equation, np);
-	} else {
-		n_tlhs = *np;
-		blt(tlhs, equation, n_tlhs * sizeof(token_type));
-		blt(save_save, jmp_save, sizeof(jmp_save));
-		if (setjmp(jmp_save) != 0) {
-			blt(jmp_save, save_save, sizeof(jmp_save));
-			/* restore the original expression */
-			*np = n_tlhs;
-			blt(equation, tlhs, n_tlhs * sizeof(token_type));
-			goto do_quick;
-		}
-		/* expand powers of 2 and higher, might result in error_huge() trap */
-		do {
-			uf_repeat(equation, np);
-		} while (uf_tsimp(equation, np));
-		blt(jmp_save, save_save, sizeof(jmp_save));
-	}
-	simpb_side(equation, np, true, 1);
-	debug_string(1, "Simplify result before applying polynomial operations:");
-	side_debug(1, equation, *np);
-	for (flag = false;;) {
-		/* divide top and bottom of fractions by any polynomial GCD found */
-		if (poly_gcd_simp(equation, np)) {
-			flag = false;
-			simpb_side(equation, np, true, 1);
-		}
-		if (!flag) {
-			/* factor polynomials */
-			if (poly_factor(equation, np)) {
-				flag = true;
-				simpb_side(equation, np, true, 1);
-				continue;
-			}
-		}
-		/* simplify algebraic fractions with polynomial and smart division */
-		if (!frac_flag && div_remainder(equation, np, poly_flag, quick_flag)) {
-			flag = false;
-			simpb_side(equation, np, true, 1);
-			continue;
-		}
-		break;
-	}
-	simpb_side(equation, np, true, 1);
-	simp_constant_power(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
-	unsimp_power(equation, np);
-	make_fractions(equation, np);
-	factor_power(equation, np);
-	uf_tsimp(equation, np);
-	make_fractions(equation, np);
-	uf_power(equation, np);
-	integer_root_simp(equation, np);
-	simpb_side(equation, np, true, 3);
-	if (!frac_flag) {
-		while (div_remainder(equation, np, poly_flag, quick_flag)) {
-			simpb_side(equation, np, true, 3);
-		}
-	}
-	poly_factor(equation, np);
-	simpb_side(equation, np, true, 2);
-}
-
-/*
- * Compare function for qsort(3) within simpb_side().
- */
-static int
-vcmp(p1, p2)
-sort_type	*p1, *p2;
-{
-	if (((p1->v & VAR_MASK) == SIGN) == ((p2->v & VAR_MASK) == SIGN)) {
-		if (p2->count == p1->count) {
-			if (p1->v < p2->v)
-				return -1;
-			if (p1->v == p2->v)
-				return 0;
-			return 1;
-		}
-		return(p2->count - p1->count);
-	} else {
-		if ((p1->v & VAR_MASK) == SIGN) {
-			return -1;
-		} else {
-			return 1;
-		}
-	}
-}
-
-/*
- * Beauty simplifier.
- * Neat simplify and factor routine.
- * Factors variables in order: "sign" variables first, then by frequency.
- */
-simpb_side(equation, np, power_flag, fc_level)
-token_type	*equation;	/* pointer to the beginning of equation side */
-int		*np;		/* pointer to length of equation side */
-int		power_flag;	/* factor_power() flag */
-int		fc_level;	/* factor constants code, passed to factor_constants() */
-{
-	int		i;
-	int		vc, cnt;
-	long		v1, last_v;
-	sort_type	va[MAX_VARS];
-
-	simp_loop(equation, np);
-	uf_allpower(equation, np);
-	last_v = 0;
-	for (vc = 0; vc < ARR_CNT(va); vc++) {
-		cnt = 0;
-		v1 = -1;
-		for (i = 0; i < *np; i += 2) {
-			if (equation[i].kind == VARIABLE && equation[i].token.variable > last_v) {
-				if (v1 == -1 || equation[i].token.variable < v1) {
-					v1 = equation[i].token.variable;
-					cnt = 1;
-				} else if (equation[i].token.variable == v1) {
-					cnt++;
-				}
-			}
-		}
-		if (v1 == -1)
-			break;
-		last_v = v1;
-		va[vc].v = v1;
-		va[vc].count = cnt;
-	}
-	if (vc) {
-		qsort((char *) va, vc, sizeof(*va), vcmp);
-		simp2_divide(equation, np, va[0].v, fc_level);
-		for (i = 1; i < vc; i++) {
-			if (factor_divide(equation, np, va[i].v, 0.0))
-				simp2_divide(equation, np, va[i].v, fc_level);
-		}
-		simp2_divide(equation, np, 0L, fc_level);
-		for (i = 0; i < vc; i++) {
-			while (factor_plus(equation, np, va[i].v, 0.0)) {
-				simp2_divide(equation, np, 0L, fc_level);
-			}
-		}
-		while (factor_plus(equation, np, MATCH_ANY, 0.0)) {
-			simp2_divide(equation, np, 0L, fc_level);
-		}
-	}
-	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
-}
-
-/*
- * The most basic simplification loop.
- * Just does constant simplification.
- */
-elim_loop(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-{
-	side_debug(6, equation, *np);
-	do {
-		do {
-			do {
-				organize(equation, np);
-			} while (combine_constants(equation, np, true));
-		} while (elim_k(equation, np));
-	} while (simp_pp(equation, np));
-	if (reorder(equation, np)) {
-		do {
-			organize(equation, np);
-		} while (elim_k(equation, np));
-	}
-	side_debug(5, equation, *np);
-}
-
-/*
- * Configurable high level simplify routine.
- */
-simp_ssub(equation, np, v, d, power_flag, times_flag, fc_level)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
-double		d;		/* factor expressions raised to the power of this if v */
-int		power_flag;	/* factor_power() flag */
-int		times_flag;	/* factor_times() flag */
-int		fc_level;	/* factor constants code, passed to factor_constants() */
-{
-	do {
-		do {
-			do {
-				do {
-					do {
-						do {
-							do {
-								do {
-									elim_loop(equation, np);
-								} while (simp2_power(equation, np));
-							} while (times_flag && factor_times(equation, np));
-						} while (elim_sign(equation, np));
-					} while (subtract_itself(equation, np));
-				} while (factor_constants(equation, np, fc_level));
-			} while (factor_divide(equation, np, v, d));
-		} while (factor_plus(equation, np, v, d));
-	} while (power_flag && factor_power(equation, np));
-}
-
-/*
- * Commonly used quick simplify routine that doesn't factor.
- *
- * Return true if factor_times() simplified something.
- */
-int
-simp_loop(equation, np)
-token_type	*equation;	/* pointer to the beginning of equation side to simplify */
-int		*np;		/* pointer to length of equation side */
-{
-	int	i;
-	int	rv = false;
-
-	do {
-		do {
-			do {
-				do {
-					elim_loop(equation, np);
-				} while (simp2_power(equation, np));
-				i = factor_times(equation, np);
-				if (i)
-					rv = true;
-			} while (i);
-		} while (elim_sign(equation, np));
-	} while (subtract_itself(equation, np));
-	return rv;
-}
-
-/*
- * Factor only the specified variable "v".
- * If v is IMAGINARY, do complex number simplification.
- */
-factorv(equation, np, v)
-token_type	*equation;	/* pointer to the beginning of equation side to factor */
-int		*np;		/* pointer to equation side length */
-long		v;		/* variable to factor */
-{
-	if (v == IMAGINARY) {
-		do {
-			elim_loop(equation, np);
-		} while (complex_root_simp(equation, np));
-	}
-	do {
-		do {
-			simp_loop(equation, np);
-		} while (factor_plus(equation, np, v, 0.0));
-	} while (v == IMAGINARY && div_imaginary(equation, np));
-}
-
-/*
- * Combine all like denominators.
- */
-simp_divide(equation, np)
-token_type	*equation;
-int		*np;
-{
-	do {
-		do {
-			simp_loop(equation, np);
-		} while (factor_constants(equation, np, 1));
-	} while (factor_divide(equation, np, 0L, 0.0));
-}
-
-/*
- * Combine all like denominators containing variable "v".
- * Don't call factor_times().
- */
-simp2_divide(equation, np, v, fc_level)
-token_type	*equation;
-int		*np;
-long		v;
-int		fc_level;
-{
-	do {
-		do {
-			do {
-				do {
-					do {
-						elim_loop(equation, np);
-					} while (simp2_power(equation, np));
-				} while (elim_sign(equation, np));
-			} while (subtract_itself(equation, np));
-		} while (factor_constants(equation, np, fc_level));
-	} while (factor_divide(equation, np, v, 0.0));
-}
-
 /*
  * Fix up levels of parentheses in an equation side.
- * Put similar operators on the same level.
+ * Remove unnecessary parentheses and put similar operators on the same level.
  *
  * This is the inner-most loop in Mathomatic, make it fast.
  */
+void
 organize(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
@@ -636,6 +144,511 @@ int		loc, level, *elocp;
 	return op;
 }
 
+/*
+ * The most basic simplification loop.
+ * Just does constant simplification.
+ */
+void
+elim_loop(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+{
+	if (abort_flag) {
+		/* Control-C pressed, return to main prompt and leave unsimplified */
+		abort_flag = false;
+		longjmp(jmp_save, 13);
+	}
+	side_debug(6, equation, *np);
+	do {
+		do {
+			do {
+				organize(equation, np);
+			} while (combine_constants(equation, np, true));
+		} while (elim_k(equation, np));
+	} while (simp_pp(equation, np));
+	if (reorder(equation, np)) {
+		do {
+			organize(equation, np);
+		} while (elim_k(equation, np));
+	}
+	side_debug(5, equation, *np);
+}
+
+/*
+ * Quickly and very basically simplify an equation space.
+ * No factoring is done.
+ */
+void
+simp_sub(n)
+int	n;	/* equation space number to simplify */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	simp_loop(lhs[n], &n_lhs[n]);
+	if (n_rhs[n]) {
+		simp_loop(rhs[n], &n_rhs[n]);
+	}
+}
+
+/*
+ * For quick, mid-range simplification of an expression.
+ */
+void
+simp_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+{
+	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
+}
+
+/*
+ * This function is the mid-range simplifier used by the solver.
+ */
+void
+simps_side(equation, np, zsolve)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+int		zsolve;		/* true for solving for zero */
+{
+	elim_loop(equation, np);
+	simp_constant_power(equation, np);
+	do {
+		simp_ssub(equation, np, 0L, 0.0, !zsolve, true, 2);
+	} while (super_factor(equation, np, 0));
+}
+
+/*
+ * This function is used by the factor command.
+ */
+void
+simpv_side(equation, np, v)
+token_type	*equation;	/* pointer to the beginning of equation side to factor */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* variable to factor, 0 for all variables */
+{
+	if (*np == 0)
+		return;
+	simp_ssub(equation, np, v, 0.0, v == 0, true, 2);
+}
+
+/*
+ * Convert expression with any algebraic fractions into a single fraction.
+ */
+void
+frac_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+{
+	if (*np == 0)
+		return;
+	do {
+		simpb_side(equation, np, false, 2);
+	} while (super_factor(equation, np, 2));
+}
+
+/*
+ * Simplify and approximate for the calculate command and for more successful comparisons.
+ * Includes complex number simplification.
+ */
+void
+calc_simp(equation, np)
+token_type	*equation;
+int		*np;
+{
+	approximate_roots = true;
+	subst_constants(equation, np);
+	simp_side(equation, np);
+	uf_power(equation, np);
+	factorv(equation, np, IMAGINARY);
+	ufactor(equation, np);
+	factorv(equation, np, IMAGINARY);
+	uf_simp(equation, np);
+	factorv(equation, np, IMAGINARY);
+	simp_side(equation, np);
+	uf_simp(equation, np);
+	approximate_roots = false;
+}
+
+/*
+ * Try to eliminate imaginary numbers (i#) from an equation side by converting
+ * expressions like (i#*(b^.5)) to ((-1*b)^.5).
+ */
+void
+simp_i(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	i;
+	int	level;
+
+	simp_loop(equation, np);
+	for (i = 0; i < *np; i += 2) {
+		if (equation[i].kind == VARIABLE && equation[i].token.variable == IMAGINARY) {
+			if (*np + 2 > n_tokens) {
+				error_huge();
+			}
+			level = equation[i].level + 1;
+			blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
+			*np += 2;
+			equation[i].level = level;
+			equation[i].kind = CONSTANT;
+			equation[i].token.constant = -1.0;
+			i++;
+			equation[i].level = level;
+			equation[i].kind = OPERATOR;
+			equation[i].token.operatr = POWER;
+			i++;
+			equation[i].level = level;
+			equation[i].kind = CONSTANT;
+			equation[i].token.constant = 0.5;
+		}
+	}
+	do {
+		do {
+			do {
+				do {
+					organize(equation, np);
+					combine_constants(equation, np, false);
+				} while (elim_k(equation, np));
+			} while (simp_pp(equation, np));
+		} while (factor_power(equation, np));
+	} while (factor_times(equation, np));
+	simp_loop(equation, np);
+}
+
+/*
+ * Main simplify code used by the simplify command.
+ * This is the slowest and most thorough simplify of all.
+ * Applies many equivalent algebraic transformations and their inverses (like unfactor and factor),
+ * then does generalized polynomial simplifications.
+ *
+ * Globals tlhs[] and trhs[] are wiped out.
+ */
+void
+simpa_side(equation, np, quick_flag, frac_flag)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
+int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
+{
+	int		i;
+	int		flag, poly_flag = true;
+	jmp_buf		save_save;
+
+	if (*np <= 1)	/* no need to simplify a single constant or variable */
+		return;
+	simp_loop(equation, np);
+	poly_factor(equation, np);
+	simpb_side(equation, np, true, 1);
+	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
+	rationalize(equation, np);
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	simp_loop(equation, np);
+	/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
+	uf_repeat(equation, np);
+	do {
+		elim_loop(equation, np);
+	} while (mod_simp(equation, np));
+	/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
+	simp_i(equation, np);
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	unsimp_power(equation, np);
+	uf_neg_help(equation, np);
+	uf_times(equation, np);
+	do {
+		do {
+			do {
+				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+			} while (poly_gcd_simp(equation, np));
+		} while (uf_power(equation, np));
+	} while (super_factor(equation, np, 2));
+	unsimp_power(equation, np);
+	uf_times(equation, np);
+	factorv(equation, np, IMAGINARY);
+	uf_pplus(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
+	uf_times(equation, np);
+	uf_pplus(equation, np);
+	factorv(equation, np, IMAGINARY);
+	uf_power(equation, np);
+	do {
+		do {
+			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
+		} while (uf_power(equation, np));
+	} while (super_factor(equation, np, 2));
+	if (quick_flag) {
+do_quick:
+		uf_tsimp(equation, np);
+	} else {
+		n_tlhs = *np;
+		blt(tlhs, equation, n_tlhs * sizeof(token_type));
+		blt(save_save, jmp_save, sizeof(jmp_save));
+		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+			blt(jmp_save, save_save, sizeof(jmp_save));
+			if (i == 13) {	/* critical error code */
+				longjmp(jmp_save, i);
+			}
+			/* an error occurred, restore the original expression */
+			*np = n_tlhs;
+			blt(equation, tlhs, n_tlhs * sizeof(token_type));
+			goto do_quick;	/* skip power expansion */
+		}
+		/* expand powers of 2 and higher, might result in error_huge() trap */
+		do {
+			uf_repeat(equation, np);
+		} while (uf_tsimp(equation, np));
+		blt(jmp_save, save_save, sizeof(jmp_save));
+	}
+	simpb_side(equation, np, true, 1);
+	debug_string(1, "Simplify result before applying polynomial operations:");
+	side_debug(1, equation, *np);
+	for (flag = false;;) {
+		/* divide top and bottom of fractions by any polynomial GCD found */
+		if (poly_gcd_simp(equation, np)) {
+			flag = false;
+			simpb_side(equation, np, true, 1);
+		}
+		if (!flag) {
+			/* factor polynomials */
+			if (poly_factor(equation, np)) {
+				flag = true;
+				simpb_side(equation, np, true, 1);
+				continue;
+			}
+		}
+		/* simplify algebraic fractions with polynomial and smart division */
+		if (!frac_flag && div_remainder(equation, np, poly_flag, quick_flag)) {
+			flag = false;
+			simpb_side(equation, np, true, 1);
+			continue;
+		}
+		break;
+	}
+	simpb_side(equation, np, true, 1);
+	simp_constant_power(equation, np);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	unsimp_power(equation, np);
+	make_fractions(equation, np);
+	factor_power(equation, np);
+	partial_flag = frac_flag;
+	uf_tsimp(equation, np);
+	partial_flag = true;
+	make_fractions(equation, np);
+	uf_power(equation, np);
+	integer_root_simp(equation, np);
+	simpb_side(equation, np, true, 3);
+	if (!frac_flag) {
+		while (div_remainder(equation, np, poly_flag, quick_flag)) {
+			simpb_side(equation, np, true, 3);
+		}
+	}
+	poly_factor(equation, np);
+	simpb_side(equation, np, true, 2);
+}
+
+/*
+ * Compare function for qsort(3) within simpb_side().
+ */
+static int
+vcmp(p1, p2)
+sort_type	*p1, *p2;
+{
+	if (((p1->v & VAR_MASK) == SIGN) == ((p2->v & VAR_MASK) == SIGN)) {
+		if (p2->count == p1->count) {
+			if (p1->v < p2->v)
+				return -1;
+			if (p1->v == p2->v)
+				return 0;
+			return 1;
+		}
+		return(p2->count - p1->count);
+	} else {
+		if ((p1->v & VAR_MASK) == SIGN) {
+			return -1;
+		} else {
+			return 1;
+		}
+	}
+}
+
+/*
+ * Beauty simplifier.
+ * Neat simplify and factor routine.
+ * Factors variables in order: "sign" variables first, then by frequency.
+ */
+simpb_side(equation, np, power_flag, fc_level)
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+int		power_flag;	/* factor_power() flag */
+int		fc_level;	/* factor constants code, passed to factor_constants() */
+{
+	int		i;
+	int		vc, cnt;
+	long		v1, last_v;
+	sort_type	va[MAX_VARS];
+
+	simp_loop(equation, np);
+	uf_allpower(equation, np);
+	last_v = 0;
+	for (vc = 0; vc < ARR_CNT(va); vc++) {
+		cnt = 0;
+		v1 = -1;
+		for (i = 0; i < *np; i += 2) {
+			if (equation[i].kind == VARIABLE && equation[i].token.variable > last_v) {
+				if (v1 == -1 || equation[i].token.variable < v1) {
+					v1 = equation[i].token.variable;
+					cnt = 1;
+				} else if (equation[i].token.variable == v1) {
+					cnt++;
+				}
+			}
+		}
+		if (v1 == -1)
+			break;
+		last_v = v1;
+		va[vc].v = v1;
+		va[vc].count = cnt;
+	}
+	if (vc) {
+		qsort((char *) va, vc, sizeof(*va), vcmp);
+		simp2_divide(equation, np, va[0].v, fc_level);
+		for (i = 1; i < vc; i++) {
+			if (factor_divide(equation, np, va[i].v, 0.0))
+				simp2_divide(equation, np, va[i].v, fc_level);
+		}
+		simp2_divide(equation, np, 0L, fc_level);
+		for (i = 0; i < vc; i++) {
+			while (factor_plus(equation, np, va[i].v, 0.0)) {
+				simp2_divide(equation, np, 0L, fc_level);
+			}
+		}
+		while (factor_plus(equation, np, MATCH_ANY, 0.0)) {
+			simp2_divide(equation, np, 0L, fc_level);
+		}
+	}
+	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
+}
+
+/*
+ * Configurable high level simplify routine.
+ */
+simp_ssub(equation, np, v, d, power_flag, times_flag, fc_level)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
+double		d;		/* factor expressions raised to the power of this if v */
+int		power_flag;	/* factor_power() flag */
+int		times_flag;	/* factor_times() flag */
+int		fc_level;	/* factor constants code, passed to factor_constants() */
+{
+	do {
+		do {
+			do {
+				do {
+					do {
+						do {
+							do {
+								do {
+									elim_loop(equation, np);
+								} while (simp2_power(equation, np));
+							} while (times_flag && factor_times(equation, np));
+						} while (elim_sign(equation, np));
+					} while (subtract_itself(equation, np));
+				} while (factor_constants(equation, np, fc_level));
+			} while (factor_divide(equation, np, v, d));
+		} while (factor_plus(equation, np, v, d));
+	} while (power_flag && factor_power(equation, np));
+}
+
+/*
+ * Commonly used quick simplify routine that doesn't factor.
+ *
+ * Return true if factor_times() simplified something.
+ */
+int
+simp_loop(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of equation side */
+{
+	int	i;
+	int	rv = false;
+
+	do {
+		do {
+			do {
+				do {
+					elim_loop(equation, np);
+				} while (simp2_power(equation, np));
+				i = factor_times(equation, np);
+				if (i)
+					rv = true;
+			} while (i);
+		} while (elim_sign(equation, np));
+	} while (subtract_itself(equation, np));
+	return rv;
+}
+
+/*
+ * Factor only the specified variable "v".
+ * If v is IMAGINARY, do complex number simplification.
+ */
+factorv(equation, np, v)
+token_type	*equation;	/* pointer to the beginning of equation side to factor */
+int		*np;		/* pointer to equation side length */
+long		v;		/* variable to factor */
+{
+	if (v == IMAGINARY) {
+		do {
+			elim_loop(equation, np);
+		} while (complex_root_simp(equation, np));
+	}
+	do {
+		do {
+			simp_loop(equation, np);
+		} while (factor_plus(equation, np, v, 0.0));
+	} while (v == IMAGINARY && div_imaginary(equation, np));
+}
+
+/*
+ * Combine all like denominators.
+ */
+simp_divide(equation, np)
+token_type	*equation;
+int		*np;
+{
+	do {
+		do {
+			simp_loop(equation, np);
+		} while (factor_constants(equation, np, 1));
+	} while (factor_divide(equation, np, 0L, 0.0));
+}
+
+/*
+ * Combine all like denominators containing variable "v".
+ * Don't call factor_times().
+ */
+simp2_divide(equation, np, v, fc_level)
+token_type	*equation;
+int		*np;
+long		v;
+int		fc_level;
+{
+	do {
+		do {
+			do {
+				do {
+					do {
+						elim_loop(equation, np);
+					} while (simp2_power(equation, np));
+				} while (elim_sign(equation, np));
+			} while (subtract_itself(equation, np));
+		} while (factor_constants(equation, np, fc_level));
+	} while (factor_divide(equation, np, v, 0.0));
+}
+
 /*
  * Convert (x^n)^m to x^(n*m).
  *
@@ -654,42 +667,44 @@ int		*np;
 	double		numerator, denominator;
 
 	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			level = equation[i].level;
-			left_level = 1;
-			for (j = i - 2; j >= 0; j -= 2) {
-				if (equation[j].level < level) {
-					left_level = equation[j].level;
-					break;
-				}
+		if (equation[i].token.operatr != POWER)
+			continue;
+		level = equation[i].level;
+		left_level = 1;
+		for (j = i - 2; j >= 0; j -= 2) {
+			if (equation[j].level < level) {
+				left_level = equation[j].level;
+				break;
 			}
-			for (j = i + 2; j < *np; j += 2) {
-				if (equation[j].level <= level) {
-					if (left_level <= equation[j].level && equation[j].token.operatr == POWER) {
-						if (!symb_flag && (i + 2) == j && equation[j-1].kind == CONSTANT) {
-							f_to_fraction(equation[j-1].token.constant, &numerator, &denominator);
+		}
+		for (j = i + 2; j < *np; j += 2) {
+			if (equation[j].level <= level) {
+				if (left_level <= equation[j].level && equation[j].token.operatr == POWER) {
+					if (!symb_flag) {
+						if ((i + 2) == j && equation[i+1].kind == CONSTANT
+						    && equation[j].level == equation[j+1].level
+						    && equation[j+1].kind == CONSTANT) {
+							f_to_fraction(equation[i+1].token.constant, &numerator, &denominator);
 							if (fmod(numerator, 2.0) == 0.0) {
-								if (equation[j].level == equation[j+1].level
-								    && equation[j+1].kind == CONSTANT) {
-									f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
-									if (fmod(denominator, 2.0) == 0.0) {
-										break;
-									}
+								f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
+								if (fmod(denominator, 2.0) == 0.0) {
+									break;
 								}
 							}
-						}
-						equation[j].token.operatr = TIMES;
-						for (k = i + 1; k < j; k++)
-							equation[k].level++;
-						diff = (level - equation[j].level) + 1;
-						level = equation[j].level;
-						for (k = j; k < *np && equation[k].level >= level; k++) {
-							equation[k].level += diff;
-						}
-						modified = true;
+						} else
+							break;
 					}
-					break;
+					equation[j].token.operatr = TIMES;
+					for (k = i + 1; k < j; k++)
+						equation[k].level++;
+					diff = (level - equation[j].level) + 1;
+					level = equation[j].level;
+					for (k = j; k < *np && equation[k].level >= level; k++) {
+						equation[k].level += diff;
+					}
+					modified = true;
 				}
+				break;
 			}
 		}
 	}
@@ -774,7 +789,7 @@ int		*np;
 	return modified;
 }
 
-static inline int
+static int
 sp2_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -835,7 +850,7 @@ int		*np;
 	return modified;
 }
 
-static inline int
+static int
 sp_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
@@ -930,7 +945,7 @@ int		iflag;		/* produce imaginary numbers flag */
  *
  * Return true if successful.
  */
-static inline int
+static int
 calc(op1p, k1p, op2, k2)
 int	*op1p;	/* pointer to operator 1, sometimes modified */
 double	*k1p;	/* pointer to operand 1 and where to store the result */
@@ -978,16 +993,22 @@ double	k2;	/* operand 2 */
 			*k1p *= k2;
 		} else {
 			if (op1 == DIVIDE) {
+				check_divide_by_zero(*k1p);
 				*k1p = k2 / *k1p;
 				*op1p = TIMES;
 			} else if (op2 == DIVIDE) {
+				check_divide_by_zero(k2);
 				*k1p = *k1p / k2;
 			}
 		}
 		break;
+	case IDIVIDE:
+		check_divide_by_zero(k2);
+		modf(*k1p / k2, k1p);
+		break;
 	case MODULUS:
 		if (k2 == 0) {
-			debug_string(0, _("Warning: modulo 0 encountered, might be considered undefined."));
+			warning(_("Modulo 0 encountered, might be considered undefined."));
 		}
 #if	false	/* If fmod() always works.  It doesn't. */
 		*k1p = fmod(*k1p, k2);
@@ -1004,12 +1025,12 @@ double	k2;	/* operand 2 */
 			break;
 		}
 		if (*k1p == 0.0 && k2 < 0.0) {
-			error(_("Divide by zero error."));
+			error(_("Divide by zero error (0 raised to negative power)."));
 			longjmp(jmp_save, 2);
 		}
 		domain_check = true;
 		if (*k1p == 0.0 && k2 == 0.0) {
-			debug_string(0, _("Warning: 0^0 encountered, might be considered undefined."));
+			warning(_("0^0 encountered, might be considered undefined."));
 			d = 1.0;	/* some people don't know 0^0 */
 		} else {
 			d = pow(*k1p, k2);
@@ -1369,6 +1390,7 @@ int		*np;		/* pointer to length of equation side */
 			if (p2->level == level && p2->kind == CONSTANT) {
 /* Replace division by a constant with times its reciprocal. */
 				f_to_fraction(p2->token.constant, &numerator, &denominator);
+				check_divide_by_zero(numerator);
 				p2->token.constant = denominator / numerator;
 				p1->token.operatr = TIMES;
 				continue;
@@ -1380,6 +1402,7 @@ int		*np;		/* pointer to length of equation side */
 			}
 			break;
 		case MODULUS:
+		case IDIVIDE:
 			if (p2->level == level && p2->kind == CONSTANT && p2->token.constant == 0.0) {
 /* Replace 0%x with 0. */
 				for (p2 = p1 + 2; p2 < ep; p2 += 2) {
@@ -1489,7 +1512,7 @@ int		n2, l2, *diff_signp;
 		}
 		switch (p1->kind) {
 		case VARIABLE:
-			if (sign_flag && (p1->token.variable & VAR_MASK) == SIGN) {
+			if (sign_cmp_flag && (p1->token.variable & VAR_MASK) == SIGN) {
 				return((p2->token.variable & VAR_MASK) == SIGN);
 			} else {
 				return(p1->token.variable == p2->token.variable);
@@ -1787,7 +1810,15 @@ int		*np;
 							modified = true;
 						}
 					} else if ((equation[i-1].token.variable & VAR_MASK) == SIGN) {
-						equation[i+1].token.constant = fmod(equation[i+1].token.constant, 2.0);
+						if (fmod(denominator, 2.0) == 1.0) {
+							/* odd root, make root (denominator) 1 */
+							numerator = fmod(numerator, 2.0);
+							/* all sign^2 now translate to 1 */
+							if (numerator != equation[i+1].token.constant) {
+								equation[i+1].token.constant = numerator;
+								modified = true;
+							}
+						}
 					}
 				}
 			}
@@ -1935,7 +1966,7 @@ int		*np;
 	return(order_recurse(equation, np, 0, 1));
 }
 
-static inline void
+static void
 swap(equation, np, level, i1, i2)
 token_type	*equation;
 int		*np;
