@@ -21,17 +21,26 @@ static int	repeat_count;
  * Return true if successful.
  */
 int
-solve(want, have)
+solve_espace(want, have)
 int	want;	/* equation number containing what to solve for */
 int	have;	/* equation number to solve */
 {
-	int	rv;
+	int	rv;	/* return value */
 
-	if (n_lhs[want] > 0) {
-		rv = solve_sub(lhs[want], n_lhs[want], lhs[have], &n_lhs[have], rhs[have], &n_rhs[have]);
+	if (n_lhs[want]) {
+		if (n_rhs[want]) {
+			/* Something in the LHS and RHS of equation number "want". */
+			error(_("This program will only solve for a single variable or for zero."));
+			rv = false;
+		} else {
+			/* Normal solve: */
+			rv = solve_sub(lhs[want], n_lhs[want], lhs[have], &n_lhs[have], rhs[have], &n_rhs[have]);
+		}
 	} else {
+		/* Solve variable was preceded by an equals sign, solve using reversed equation sides: */
 		rv = solve_sub(rhs[want], n_rhs[want], rhs[have], &n_rhs[have], lhs[have], &n_lhs[have]);
 	}
+	/* Free the "want" equation space: */
 	n_lhs[want] = 0;
 	n_rhs[want] = 0;
 #if	!SILENT
@@ -49,7 +58,7 @@ int	have;	/* equation number to solve */
  * to the LHS (via transposition), then moving everything not containing the variable to the
  * RHS.  Many tricks are used, and this routine works very well.
  *
- * tlhs and trhs are used by this routine.
+ * Globals tlhs[] and trhs[] are used to hold the actual equation while manipulating.
  *
  * Returns a positive integer if successful, with the result placed in the passed LHS and RHS.
  * Returns 1 for normal success.
@@ -61,11 +70,11 @@ int	have;	/* equation number to solve */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
 token_type	*wantp;		/* expression to solve for */
-int		wantn;		/* size of expression to solve for (currently should always be 1) */
+int		wantn;		/* length of expression to solve for (currently should always be 1) */
 token_type	*leftp;		/* LHS of equation */
-int		*leftnp;	/* size of LHS */
+int		*leftnp;	/* pointer to length of LHS */
 token_type	*rightp;	/* RHS of equation */
-int		*rightnp;	/* size of RHS */
+int		*rightnp;	/* pointer to length of RHS */
 {
 	int		i, j;
 	int		found, found_count;
@@ -73,7 +82,7 @@ int		*rightnp;	/* size of RHS */
 	int		diff_sign;
 	int		op, op_kind;
 	token_type	*p1, *b1, *ep;
-	long		v;
+	long		v;			/* variable to solve for */
 	int		need_flip;
 	int		uf_flag = false;	/* unfactor flag */
 	int		qtries = 0;
@@ -101,6 +110,7 @@ int		*rightnp;	/* size of RHS */
 	if (wantp->kind == VARIABLE) {
 		v = wantp->token.variable;
 		if (!found_var(trhs, n_trhs, v) && !found_var(tlhs, n_tlhs, v)) {
+			/* variable v is 0 or not found */
 			error(_("Variable not found."));
 			return false;
 		}
@@ -196,84 +206,77 @@ left_again:
 	worked = true;
 	uf_flag = false;
 see_work:
+	if (found_var(trhs, n_trhs, v)) {
+		/* solve variable in RHS */
+		debug_string(1, _("Solve variable moved back to RHS, quitting solve routine."));
+		return false;
+	}
 	/* See if we have solved the equation. */
-	if (se_compare(wantp, wantn, tlhs, n_tlhs, &diff_sign)) {
-		if (!diff_sign) {
-			if (!zsolve) {
-				ep = &trhs[n_trhs];
-				for (p1 = trhs; p1 < ep; p1 += 2) {
-					if (p1->kind == VARIABLE && p1->token.variable == v) {
-						return false;
+	if (se_compare(wantp, wantn, tlhs, n_tlhs, &diff_sign) && !diff_sign) {
+		if (zsolve) {
+zero_simp:
+			list_tdebug(4);
+			uf_power(trhs, &n_trhs);
+			do {
+				do {
+					simp_ssub(trhs, &n_trhs, 0L, 0.0, false, true, 0);
+				} while (uf_power(trhs, &n_trhs));
+			} while (super_factor(trhs, &n_trhs, 1));
+			list_tdebug(3);
+			ep = &trhs[n_trhs];
+			op = 0;
+			for (p1 = trhs + 1; p1 < ep; p1 += 2) {
+				if (p1->level == 1) {
+					op = p1->token.operatr;
+					if (op == DIVIDE) {
+						goto no_simp;
+					}
+					if (op != TIMES) {
+						break;
 					}
 				}
-			} else {
-zero_simp:
-				list_tdebug(4);
-				uf_power(trhs, &n_trhs);
-				do {
-					do {
-						simp_ssub(trhs, &n_trhs, 0L, 0.0, false, true, 0);
-					} while (uf_power(trhs, &n_trhs));
-				} while (super_factor(trhs, &n_trhs, 1));
-				list_tdebug(3);
-				ep = &trhs[n_trhs];
-				op = 0;
-				for (p1 = &trhs[1]; p1 < ep; p1 += 2) {
-					if (p1->level == 1) {
-						op = p1->token.operatr;
-						if (op == DIVIDE) {
-							goto no_simp;
+			}
+			switch (op) {
+			case TIMES:
+				for (p1 = trhs; p1 < ep;) {
+					b1 = p1;
+					for (;; p1++) {
+						if (p1 >= ep || (p1->kind == OPERATOR && p1->level == 1)) {
+							blt(b1 + 1, p1, (char *) ep - (char *) p1);
+							n_trhs -= p1 - (b1 + 1);
+							b1->kind = CONSTANT;
+							b1->token.constant = 1.0;
+							goto zero_simp;
 						}
-						if (op != TIMES) {
+						if (p1->kind != CONSTANT && p1->kind != OPERATOR
+						    && (p1->kind != VARIABLE || (p1->token.variable & VAR_MASK) > SIGN)) {
 							break;
 						}
 					}
+					p1 = b1;
+					for (p1++; p1 < ep && p1->level > 1; p1 += 2)
+						;
+					p1++;
 				}
-				switch (op) {
-				case TIMES:
-					for (p1 = trhs; p1 < ep;) {
-						b1 = p1;
-						for (;; p1++) {
-							if (p1 >= ep || (p1->kind == OPERATOR && p1->level == 1)) {
-								blt(b1 + 1, p1, (char *) ep - (char *) p1);
-								n_trhs -= p1 - (b1 + 1);
-								b1->kind = CONSTANT;
-								b1->token.constant = 1.0;
-								goto zero_simp;
-							}
-							if (p1->kind != CONSTANT && p1->kind != OPERATOR
-							    && (p1->kind != VARIABLE || (p1->token.variable & VAR_MASK) > SIGN)) {
-								break;
-							}
-						}
-						p1 = b1;
-						for (p1++; p1 < ep && p1->level > 1; p1 += 2)
-							;
-						p1++;
-					}
-					break;
-				case POWER:
-					if ((p1 + 1)->level == 1
-					    && (p1 + 1)->kind == CONSTANT
-					    && (p1 + 1)->token.constant > 0.0) {
-						n_trhs -= 2;
-						goto zero_simp;
-					}
-					break;
+				break;
+			case POWER:
+				p1++;
+				if (p1->level == 1 && p1->kind == CONSTANT && p1->token.constant > 0.0) {
+					n_trhs -= 2;
+					goto zero_simp;
 				}
+				break;
 			}
-			if (zsolve) {
-				debug_string(1, _("Solve for zero completed:"));
-			} else {
-				debug_string(1, _("Solve completed:"));
-			}
-			list_tdebug(1);
-			blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
-			*leftnp = n_tlhs;
-			blt(rightp, trhs, n_trhs * sizeof(*rightp));
-			*rightnp = n_trhs;
-			return success;
+			debug_string(1, _("Solve for zero completed:"));
+		} else {
+			debug_string(1, _("Solve completed:"));
 		}
+		list_tdebug(1);
+		blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
+		*leftnp = n_tlhs;
+		blt(rightp, trhs, n_trhs * sizeof(*rightp));
+		*rightnp = n_trhs;
+		return success;
 	}
 	/* Move what we don't want in the LHS to the RHS. */
 	found_count = 0;
@@ -347,12 +350,12 @@ zero_simp:
 			}
 			if (p1 >= ep) {
 				if (found_count == 0) { /* if solve variable no longer in LHS */
-					for (i = 0; i < n_trhs; i += 2) {
-						if (trhs[i].kind == VARIABLE && trhs[i].token.variable == v) {
-							/* solve variable in RHS */
-							return false;
-						}
+					if (found_var(trhs, n_trhs, v)) {
+						/* solve variable in RHS */
+						debug_string(1, _("Solve variable moved back to RHS, quitting solve routine."));
+						return false;
 					}
+					/* The following code determines if we have an identity: */
 					calc_simp(tlhs, &n_tlhs);
 					calc_simp(trhs, &n_trhs);
 					if (se_compare(tlhs, n_tlhs, trhs, n_trhs, &diff_sign) && !diff_sign) {
@@ -364,15 +367,13 @@ zero_simp:
 					}
 					found = false;
 					for (i = 0; i < n_tlhs; i += 2) {
-						if (tlhs[i].kind == VARIABLE
-						    && tlhs[i].token.variable > IMAGINARY) {
+						if (tlhs[i].kind == VARIABLE && tlhs[i].token.variable > IMAGINARY) {
 							found = true;
 							break;
 						}
 					}
 					for (i = 0; i < n_trhs; i += 2) {
-						if (trhs[i].kind == VARIABLE
-						    && trhs[i].token.variable > IMAGINARY) {
+						if (trhs[i].kind == VARIABLE && trhs[i].token.variable > IMAGINARY) {
 							found = true;
 							break;
 						}
@@ -574,10 +575,9 @@ long	v;
 	simp_ssub(tlhs, &n_tlhs, 0L, 0.0, true, true, 2);
 lonely:
 	ep = &tlhs[n_tlhs];
-	len1 = 0;
-	len2 = 0;
+	len2 = len1 = 0;
 	foundp = false;
-	for (p1 = &tlhs[1];; p1 += 2) {
+	for (p1 = tlhs + 1;; p1 += 2) {
 		if (p1 >= ep) {
 			return false;
 		}
@@ -1032,7 +1032,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	oldn = *side1np;
 	ep = &side1p[oldn];
 	if (*side1np == prev_n1 && *side2np == prev_n2) {
-		if (++repeat_count >= 5) {
+		if (++repeat_count >= 3) {
 			debug_string(1, _("Infinite loop aborted in solve routine."));
 			return false;
 		}
@@ -1082,6 +1082,14 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		*side1np -= operandn + 1;
 		return true;
 	}
+	if (op == MODULUS) {
+		if (get_constant(side2p, *side2np, &d1) && get_constant(operandp, operandn, &d2)) {
+			if (fabs(d1) >= fabs(d2)) {
+				error(_("There are no possible solutions."));
+				return false;
+			}
+		}
+	}
 #if	!SILENT
 	if (debug_level > 0) {
 		switch (op) {
