@@ -74,9 +74,9 @@ int		*np, loc, level;
 }
 
 /*
- * Group denominators together.
- * Grouping here means converting "a/b/c/d*e" to "a*e/(b*c*d)" or "a/(b*c*d)*e",
- * not guaranteed to put the grouped divisors last, reorder() puts divisors last.
+ * Group denominators together in an equation side.
+ * Grouping here means converting "a/b/c/d*e" to "a*e/(b*c*d)" or "a/(b*c*d)*e".
+ * Not guaranteed to put the grouped divisors last, reorder() puts divisors last.
  */
 void
 group_proc(equation, np)
@@ -91,7 +91,7 @@ int		*np;
  */
 void
 make_fractions_and_group(n)
-int	n;
+int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return;
@@ -106,6 +106,21 @@ int	n;
 	}
 }
 
+/*
+ * Make equation side ready for display.
+ */
+void
+display_fractions_and_group(equation, np)
+token_type	*equation;
+int		*np;
+{
+	if (display2d) {
+		elim_loop(equation, np);
+		make_fractions(equation, np);
+		group_proc(equation, np);
+	}
+}
+
 /*
  * Combine fractions by making the denominators all the same.
  * This means converting "a/b+c/d" to "(a*d+c*b)/b/d".
