@@ -162,35 +162,17 @@ int		*np;
 	return rv;
 }
 
-static inline void
+static void
 no_divide(equation, np)
 token_type	*equation;
 int		*np;
 {
 	int	i, j;
 	int	level;
-	int	plus_flag;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == DIVIDE) {
 			level = equation[i].level;
-#if	false
-			if (partial_flag != 2) {
-				plus_flag = false;
-				for (j = i + 2; j < *np; j += 2) {
-					if (equation[j].level <= level)
-						break;
-					switch (equation[j].token.operatr) {
-					case PLUS:
-					case MINUS:
-						plus_flag = true;
-						break;
-					}
-				}
-				if (!plus_flag)
-					continue;
-			}
-#endif
 			for (j = i - 1; j >= 0; j--) {
 				if (equation[j].level < level)
 					break;
@@ -502,7 +484,7 @@ do_repeat:
 	return false;
 }
 
-static inline int
+static int
 usp_sub(equation, np, i)
 token_type	*equation;
 int		*np, i;
